@@ -18,53 +18,50 @@
 
 import os
 import stat
-from typing import List, Dict, Type
 
-
-__version__ = (0, 0, 17)
-version_string = '.'.join(map(str, __version__))
+__version__ = (0, 0, 20)
+version_string = ".".join(map(str, __version__))
 
 
 USER_AGENT = f"Ognibuild/{version_string}"
 
 
 class DetailedFailure(Exception):
-    def __init__(self, retcode, argv, error):
+    def __init__(self, retcode, argv, error) -> None:
         self.retcode = retcode
         self.argv = argv
         self.error = error
 
     def __eq__(self, other):
-        return (isinstance(other, type(self)) and
-                self.retcode == other.retcode and
-                self.argv == other.argv and
-                self.error == other.error)
+        return (
+            isinstance(other, type(self))
+            and self.retcode == other.retcode
+            and self.argv == other.argv
+            and self.error == other.error
+        )
 
 
 class UnidentifiedError(Exception):
     """An unidentified error."""
 
-    def __init__(self, retcode, argv, lines, secondary=None):
+    def __init__(self, retcode, argv, lines, secondary=None) -> None:
         self.retcode = retcode
         self.argv = argv
         self.lines = lines
         self.secondary = secondary
 
     def __eq__(self, other):
-        return (isinstance(other, type(self)) and
-                self.retcode == other.retcode and
-                self.argv == other.argv and
-                self.lines == other.lines and
-                self.secondary == other.secondary)
-
-    def __repr__(self):
-        return "<%s(%r, %r, ..., secondary=%r)>" % (
-            type(self).__name__,
-            self.retcode,
-            self.argv,
-            self.secondary,
+        return (
+            isinstance(other, type(self))
+            and self.retcode == other.retcode
+            and self.argv == other.argv
+            and self.lines == other.lines
+            and self.secondary == other.secondary
         )
 
+    def __repr__(self) -> str:
+        return f"<{type(self).__name__}({self.retcode!r}, {self.argv!r}, ..., secondary={self.secondary!r})>"
+
 
 def shebang_binary(p):
     if not (os.stat(p).st_mode & stat.S_IEXEC):
@@ -80,18 +77,17 @@ def shebang_binary(p):
 
 
 class UnknownRequirementFamily(Exception):
-    """Requirement family is unknown"""
+    """Requirement family is unknown."""
 
-    def __init__(self, family):
+    def __init__(self, family) -> None:
         self.family = family
 
 
 class Requirement:
-
     # Name of the family of requirements - e.g. "python-package"
     family: str
 
-    _JSON_DESERIALIZERS: Dict[str, Type["Requirement"]] = {}
+    _JSON_DESERIALIZERS: dict[str, type["Requirement"]] = {}
 
     @classmethod
     def _from_json(self, js):
@@ -120,23 +116,22 @@ class Requirement:
 
 
 class OneOfRequirement(Requirement):
+    elements: list[Requirement]
 
-    elements: List[Requirement]
+    family = "or"
 
-    family = 'or'
-
-    def __init__(self, elements):
+    def __init__(self, elements) -> None:
         self.elements = elements
 
     def met(self, session):
         return any(req.met(session) for req in self.elements)
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.elements)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.elements!r})"
 
 
 class UpstreamOutput:
-    def __init__(self, family):
+    def __init__(self, family) -> None:
         self.family = family
 
     def get_declared_dependencies(self):
