@@ -15,73 +15,80 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Convert problems found in the buildlog to upstream requirements.
-"""
+"""Convert problems found in the buildlog to upstream requirements."""
 
 import logging
-from typing import Optional, List, cast
+from typing import Optional, cast
 
 from buildlog_consultant.common import (
-    Problem,
-    MissingPerlFile,
-    MissingSetupPyCommand,
-    MissingCMakeComponents,
-    MissingXfceDependency,
-    MissingHaskellDependencies,
-    MissingMavenArtifacts,
-    MissingGnomeCommonDependency,
-    MissingPerlPredeclared,
-    MissingLatexFile,
     MissingCargoCrate,
+    MissingCMakeComponents,
+    MissingGnomeCommonDependency,
+    MissingHaskellDependencies,
+    MissingLatexFile,
+    MissingMavenArtifacts,
+    MissingPerlFile,
+    MissingPerlPredeclared,
+    MissingSetupPyCommand,
+    MissingXfceDependency,
+    Problem,
 )
 
 from . import OneOfRequirement
+from .buildlog_converters import PROBLEM_CONVERTERS  # type: ignore
 from .fix_build import BuildFixer
 from .requirements import (
-    Requirement,
     BinaryRequirement,
-    HaskellPackageRequirement,
-    MavenArtifactRequirement,
     BoostComponentRequirement,
-    KF5ComponentRequirement,
-    PerlFileRequirement,
-    PythonPackageRequirement,
-    PerlPreDeclaredRequirement,
-    LatexPackageRequirement,
     CargoCrateRequirement,
+    HaskellPackageRequirement,
+    KF5ComponentRequirement,
+    LatexPackageRequirement,
+    MavenArtifactRequirement,
+    PerlFileRequirement,
+    PerlPreDeclaredRequirement,
+    PythonPackageRequirement,
+    Requirement,
 )
 from .resolver import UnsatisfiedRequirements
-from .buildlog_converters import PROBLEM_CONVERTERS  # type: ignore
 
 
-def problem_to_upstream_requirement(
-        problem: Problem) -> Optional[Requirement]:  # noqa: C901
+def problem_to_upstream_requirement(problem: Problem) -> Optional[Requirement]:  # noqa: C901
     for entry in PROBLEM_CONVERTERS:
         kind, fn = entry[:2]
         if kind == problem.kind:
             return fn(problem)
     if isinstance(problem, MissingCMakeComponents):
-        if problem.name.lower() == 'boost':
+        if problem.name.lower() == "boost":
             return OneOfRequirement(
-                [BoostComponentRequirement(name)
-                 for name in problem.components])
-        elif problem.name.lower() == 'kf5':
+                [
+                    BoostComponentRequirement(name)
+                    for name in problem.components
+                ]
+            )
+        elif problem.name.lower() == "kf5":
             return OneOfRequirement(
-                [KF5ComponentRequirement(name) for name in problem.components])
+                [KF5ComponentRequirement(name) for name in problem.components]
+            )
         return None
     elif isinstance(problem, MissingLatexFile):
-        if problem.filename.endswith('.sty'):
+        if problem.filename.endswith(".sty"):
             return LatexPackageRequirement(problem.filename[:-4])
         return None
     elif isinstance(problem, MissingHaskellDependencies):
         return OneOfRequirement(
-            [HaskellPackageRequirement.from_string(dep)
-             for dep in problem.deps])
+            [
+                HaskellPackageRequirement.from_string(dep)
+                for dep in problem.deps
+            ]
+        )
     elif isinstance(problem, MissingMavenArtifacts):
-        return OneOfRequirement([
-            MavenArtifactRequirement.from_str(artifact)
-            for artifact in problem.artifacts
-        ])
+        return OneOfRequirement(
+            [
+                MavenArtifactRequirement.from_str(artifact)
+                for artifact in problem.artifacts
+            ]
+        )
     elif isinstance(problem, MissingPerlPredeclared):
         ret = PerlPreDeclaredRequirement(problem.name)
         try:
@@ -101,7 +108,7 @@ def problem_to_upstream_requirement(
         else:
             logging.warning(
                 "No known command for gnome-common dependency %s",
-                problem.package
+                problem.package,
             )
             return None
     elif isinstance(problem, MissingXfceDependency):
@@ -109,31 +116,33 @@ def problem_to_upstream_requirement(
             return BinaryRequirement("gtkdocize")
         else:
             logging.warning(
-                "No known command for xfce dependency %s", problem.package)
+                "No known command for xfce dependency %s", problem.package
+            )
             return None
     elif isinstance(problem, MissingPerlFile):
         return PerlFileRequirement(filename=problem.filename)
-    elif problem.kind == 'unsatisfied-apt-dependencies':
+    elif problem.kind == "unsatisfied-apt-dependencies":
         from buildlog_consultant.apt import UnsatisfiedAptDependencies
+
         from .resolver.apt import AptRequirement
+
         return AptRequirement(
-            cast(UnsatisfiedAptDependencies, problem).relations)
+            cast(UnsatisfiedAptDependencies, problem).relations
+        )
     else:
-        logging.warning(
-            'Unable to determine how to deal with %r',
-            problem)
+        logging.warning("Unable to determine how to deal with %r", problem)
         return None
 
 
 class InstallFixer(BuildFixer):
-    def __init__(self, resolver):
+    def __init__(self, resolver) -> None:
         self.resolver = resolver
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.resolver)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.resolver!r})"
 
-    def __str__(self):
-        return "upstream requirement fixer(%s)" % self.resolver
+    def __str__(self) -> str:
+        return f"upstream requirement fixer({self.resolver})"
 
     def can_fix(self, error):
         req = problem_to_upstream_requirement(error)
@@ -144,7 +153,7 @@ class InstallFixer(BuildFixer):
         if req is None:
             return False
 
-        reqs: List[Requirement]
+        reqs: list[Requirement]
         if not isinstance(req, list):
             reqs = [req]
         else:
@@ -158,19 +167,19 @@ class InstallFixer(BuildFixer):
 
 
 class ExplainInstall(Exception):
-    def __init__(self, commands):
+    def __init__(self, commands) -> None:
         self.commands = commands
 
 
 class ExplainInstallFixer(BuildFixer):
-    def __init__(self, resolver):
+    def __init__(self, resolver) -> None:
         self.resolver = resolver
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.resolver)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.resolver!r})"
 
-    def __str__(self):
-        return "upstream requirement install explainer(%s)" % self.resolver
+    def __str__(self) -> str:
+        return f"upstream requirement install explainer({self.resolver})"
 
     def can_fix(self, error):
         req = problem_to_upstream_requirement(error)
