@@ -16,27 +16,31 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from debian.changelog import Version
 import logging
-from typing import List, Optional, Iterable
-
 import os
+from collections.abc import Iterable
+from typing import TYPE_CHECKING, Optional
+
 from buildlog_consultant.apt import (
     find_apt_get_failure,
 )
+from debian.changelog import Version
 
 from .. import DetailedFailure, UnidentifiedError
-from ..session import Session, run_with_tee, get_user
+from ..session import Session, get_user, run_with_tee
 from .file_search import (
+    GENERATED_FILE_SEARCHER,
     FileSearcher,
     get_apt_contents_file_searcher,
-    GENERATED_FILE_SEARCHER,
     get_packages_for_paths,
 )
 
+if TYPE_CHECKING:
+    import apt
+
 
 def run_apt(
-    session: Session, args: List[str], prefix: Optional[List[str]] = None
+    session: Session, args: list[str], prefix: Optional[list[str]] = None
 ) -> None:
     """Run apt.
 
@@ -47,24 +51,24 @@ def run_apt(
     if prefix is None:
         prefix = []
     args = prefix = ["apt", "-y"] + args
-    logging.info('apt: running %r', args)
+    logging.info("apt: running %r", args)
     retcode, lines = run_with_tee(session, args, cwd="/", user="root")
     if retcode == 0:
         return
     match, error = find_apt_get_failure(lines)
     if error is not None:
         raise DetailedFailure(retcode, args, error)
-    while lines and lines[-1].rstrip('\n') == "":
+    while lines and lines[-1].rstrip("\n") == "":
         lines.pop(-1)
     raise UnidentifiedError(retcode, args, lines, secondary=match)
 
 
 class AptManager:
-
     session: Session
-    _searchers: Optional[List[FileSearcher]]
+    _searchers: Optional[list[FileSearcher]]
+    _apt_cache: Optional["apt.Cache"]
 
-    def __init__(self, session, prefix=None):
+    def __init__(self, session, prefix=None) -> None:
         self.session = session
         self._apt_cache = None
         self._searchers = None
@@ -106,11 +110,14 @@ class AptManager:
             return None
 
     async def get_packages_for_paths(
-            self, paths, regex: bool = False, case_insensitive: bool = False):
+        self, paths, regex: bool = False, case_insensitive: bool = False
+    ):
         logging.debug("Searching for packages containing %r", paths)
         return await get_packages_for_paths(
-            paths, self.searchers(), regex=regex,
-            case_insensitive=case_insensitive
+            paths,
+            self.searchers(),
+            regex=regex,
+            case_insensitive=case_insensitive,
         )
 
     def missing(self, packages):
@@ -130,14 +137,14 @@ class AptManager:
                         missing.remove(section["Package"])
         return list(missing)
 
-    def install(self, packages: List[str]) -> None:
+    def install(self, packages: list[str]) -> None:
         logging.info("Installing using apt: %r", packages)
         packages = self.missing(packages)
         if packages:
             run_apt(self.session, ["install"] + packages, prefix=self.prefix)
 
-    def satisfy(self, deps: List[str]) -> None:
+    def satisfy(self, deps: list[str]) -> None:
         run_apt(self.session, ["satisfy"] + deps, prefix=self.prefix)
 
-    def satisfy_command(self, deps: List[str]) -> List[str]:
+    def satisfy_command(self, deps: list[str]) -> list[str]:
         return self.prefix + ["apt", "satisfy"] + deps
