@@ -16,17 +16,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import apt_pkg
 import asyncio
-from contextlib import suppress
-from datetime import datetime
-from debian.deb822 import Release
+import logging
 import os
 import re
 import subprocess
-from typing import List, AsyncIterator
-import logging
+from collections.abc import AsyncIterator
+from contextlib import suppress
+from datetime import datetime
 
+import apt_pkg
+from debian.deb822 import Release
 
 from .. import USER_AGENT
 from ..session import Session
@@ -34,8 +34,8 @@ from ..session import Session
 
 class FileSearcher:
     def search_files(
-            self, path: str, regex: bool = False,
-            case_insensitive: bool = False) -> AsyncIterator[str]:
+        self, path: str, regex: bool = False, case_insensitive: bool = False
+    ) -> AsyncIterator[str]:
         raise NotImplementedError(self.search_files)
 
 
@@ -68,17 +68,19 @@ def contents_urls_from_sources_entry(source, arches, load_url):
         dists_url = base_url + "/dists"
     else:
         dists_url = base_url
-    inrelease_url = "%s/%s/InRelease" % (dists_url, name)
+    inrelease_url = f"{dists_url}/{name}/InRelease"
     try:
         response = load_url(inrelease_url)
     except FileNotFoundError:
-        release_url = "%s/%s/Release" % (dists_url, name)
+        release_url = f"{dists_url}/{name}/Release"
         try:
             response = load_url(release_url)
         except FileNotFoundError as e:
             logging.warning(
-                "Unable to download %s or %s: %s", inrelease_url,
-                release_url, e
+                "Unable to download %s or %s: %s",
+                inrelease_url,
+                release_url,
+                e,
             )
             return
 
@@ -92,14 +94,14 @@ def contents_urls_from_sources_entry(source, arches, load_url):
     if components:
         for component in components:
             for arch in arches:
-                contents_files.add("%s/Contents-%s" % (component, arch))
+                contents_files.add(f"{component}/Contents-{arch}")
     else:
         for arch in arches:
-            contents_files.add("Contents-%s" % (arch,))
+            contents_files.add(f"Contents-{arch}")
 
     for fn in contents_files:
         if fn in existing_names:
-            url = "%s/%s/%s" % (dists_url, name, fn)
+            url = f"{dists_url}/{name}/{fn}"
             yield url
 
 
@@ -126,7 +128,7 @@ def _unwrap(f, ext):
 
 def load_direct_url(url):
     from urllib.error import HTTPError, URLError
-    from urllib.request import urlopen, Request
+    from urllib.request import Request, urlopen
 
     for ext in [".xz", ".gz", ""]:
         try:
@@ -136,10 +138,12 @@ def load_direct_url(url):
             if e.code == 404:
                 continue
             raise AptFileAccessError(
-                'Unable to access apt URL %s: %s' % (url + ext, e)) from e
+                f"Unable to access apt URL {url + ext}: {e}"
+            ) from e
         except URLError as e:
             raise AptFileAccessError(
-                'Unable to access apt URL %s: %s' % (url + ext, e)) from e
+                f"Unable to access apt URL {url + ext}: {e}"
+            ) from e
         break
     else:
         raise FileNotFoundError(url)
@@ -166,19 +170,18 @@ def load_apt_cache_file(url, cache_dir):
 
             return lz4.frame.open(p, mode="rb")
         try:
-            f = open(p, "rb")   # noqa: SIM115
+            f = open(p, "rb")  # noqa: SIM115
         except PermissionError as e:
-            logging.warning('Unable to open %s: %s', p, e)
+            logging.warning("Unable to open %s: %s", p, e)
             raise FileNotFoundError(url) from e
         return _unwrap(f, ext)
     raise FileNotFoundError(url)
 
 
 class AptFileFileSearcher(FileSearcher):
+    CACHE_IS_EMPTY_PATH = "/usr/share/apt-file/is-cache-empty"
 
-    CACHE_IS_EMPTY_PATH = '/usr/share/apt-file/is-cache-empty'
-
-    def __init__(self, session: Session):
+    def __init__(self, session: Session) -> None:
         self.session = session
 
     @classmethod
@@ -196,39 +199,42 @@ class AptFileFileSearcher(FileSearcher):
 
     @classmethod
     def from_session(cls, session):
-        logging.debug('Using apt-file to search apt contents')
+        logging.debug("Using apt-file to search apt contents")
         if not os.path.exists(session.external_path(cls.CACHE_IS_EMPTY_PATH)):
             from .apt import AptManager
-            AptManager.from_session(session).install(['apt-file'])
+
+            AptManager.from_session(session).install(["apt-file"])
         if not cls.has_cache(session):
-            session.check_call(['apt-file', 'update'], user='root')
+            session.check_call(["apt-file", "update"], user="root")
         return cls(session)
 
     async def search_files(self, path, regex=False, case_insensitive=False):
         args = []
         if regex:
-            args.append('-x')
+            args.append("-x")
         else:
-            args.append('-F')
+            args.append("-F")
         if case_insensitive:
-            args.append('-i')
+            args.append("-i")
         args.append(path)
         process = await asyncio.create_subprocess_exec(
-            '/usr/bin/apt-file', 'search', *args,
-            stdout=asyncio.subprocess.PIPE)
+            "/usr/bin/apt-file",
+            "search",
+            *args,
+            stdout=asyncio.subprocess.PIPE,
+        )
         (output, error) = await process.communicate(input=None)
         if process.returncode == 1:
             # No results
             return
         elif process.returncode == 3:
-            raise Exception('apt-file cache is empty')
+            raise Exception("apt-file cache is empty")
         elif process.returncode != 0:
-            raise Exception(
-                 "unexpected return code %r" % process.returncode)
+            raise Exception(f"unexpected return code {process.returncode!r}")
 
         for line in output.splitlines(False):
-            pkg, path = line.split(b': ')
-            yield pkg.decode('utf-8')
+            pkg, path = line.split(b": ")
+            yield pkg.decode("utf-8")
 
 
 def get_apt_contents_file_searcher(session):
@@ -239,8 +245,8 @@ def get_apt_contents_file_searcher(session):
 
 
 class RemoteContentsFileSearcher(FileSearcher):
-    def __init__(self):
-        self._db = {}
+    def __init__(self) -> None:
+        self._db: dict[str, bytes] = {}
 
     @classmethod
     def from_session(cls, session):
@@ -259,14 +265,15 @@ class RemoteContentsFileSearcher(FileSearcher):
 
         from .build import get_build_architecture
 
-        cache_dirs = set(["/var/lib/apt/lists"])
+        cache_dirs = {"/var/lib/apt/lists"}
 
         def load_url(url):
             return load_url_with_cache(url, cache_dirs)
 
         urls = list(
             contents_urls_from_sourceslist(
-                sl, get_build_architecture(), load_url)
+                sl, get_build_architecture(), load_url
+            )
         )
         self._load_urls(urls, cache_dirs, load_url)
 
@@ -279,19 +286,19 @@ class RemoteContentsFileSearcher(FileSearcher):
 
         from .build import get_build_architecture
 
-        cache_dirs = set(
-            [
-                os.path.join(session.location, "var/lib/apt/lists"),
-                "/var/lib/apt/lists",
-            ]
-        )
+        cache_dirs = {
+            os.path.join(session.location, "var/lib/apt/lists"),
+            "/var/lib/apt/lists",
+        }
 
         def load_url(url):
             return load_url_with_cache(url, cache_dirs)
 
         urls = list(
             contents_urls_from_sourceslist(
-                sl, get_build_architecture(), load_url))
+                sl, get_build_architecture(), load_url
+            )
+        )
         self._load_urls(urls, cache_dirs, load_url)
 
     def _load_urls(self, urls, cache_dirs, load_url):
@@ -305,7 +312,7 @@ class RemoteContentsFileSearcher(FileSearcher):
             except ContentsFileNotFound:
                 logging.warning("Unable to fetch contents file %s", url)
 
-    def __setitem__(self, path, package):
+    def __setitem__(self, path, package) -> None:
         self._db[path] = package
 
     async def search_files(self, path, regex=False, case_insensitive=False):
@@ -323,8 +330,8 @@ class RemoteContentsFileSearcher(FileSearcher):
                 if c.match(p):
                     pkg = rest.split(b"/")[-1]
                     ret.append((p, pkg.decode("utf-8")))
-            for _p, pkg in sorted(ret):
-                yield pkg
+            for _p, package in sorted(ret):
+                yield package
         else:
             with suppress(KeyError):
                 yield self._db[path].split(b"/")[-1].decode("utf-8")
@@ -337,7 +344,7 @@ class RemoteContentsFileSearcher(FileSearcher):
 
 
 class GeneratedFileSearcher(FileSearcher):
-    def __init__(self, db):
+    def __init__(self, db) -> None:
         self._db = db
 
     @classmethod
@@ -347,14 +354,14 @@ class GeneratedFileSearcher(FileSearcher):
         return self
 
     def load_from_path(self, path):
-        with open(path, "r") as f:
+        with open(path) as f:
             for line in f:
                 (path, pkg) = line.strip().split(None, 1)
                 self._db.append(path, pkg)
 
     async def search_files(
-            self, path: str, regex: bool = False,
-            case_insensitive: bool = False):
+        self, path: str, regex: bool = False, case_insensitive: bool = False
+    ):
         for p, pkg in self._db:
             if regex:
                 flags = 0
@@ -386,12 +393,12 @@ GENERATED_FILE_SEARCHER = GeneratedFileSearcher(
 
 
 async def get_packages_for_paths(
-    paths: List[str],
-    searchers: List[FileSearcher],
+    paths: list[str],
+    searchers: list[FileSearcher],
     regex: bool = False,
     case_insensitive: bool = False,
-) -> List[str]:
-    candidates: List[str] = list()
+) -> list[str]:
+    candidates: list[str] = list()
     # TODO(jelmer): Combine these, perhaps by creating one gigantic regex?
     for path in paths:
         for searcher in searchers:
@@ -405,13 +412,16 @@ async def get_packages_for_paths(
 
 def main(argv):
     import argparse
+
     from ..session.plain import PlainSession
 
     parser = argparse.ArgumentParser()
     parser.add_argument(
-        "path", help="Path to search for.", type=str, nargs="*")
+        "path", help="Path to search for.", type=str, nargs="*"
+    )
     parser.add_argument(
-        "--regex", "-x", help="Search for regex.", action="store_true")
+        "--regex", "-x", help="Search for regex.", action="store_true"
+    )
     parser.add_argument("--debug", action="store_true")
     args = parser.parse_args()
 
@@ -424,8 +434,11 @@ def main(argv):
         main_searcher = get_apt_contents_file_searcher(session)
         searchers = [main_searcher, GENERATED_FILE_SEARCHER]
 
-        packages = asyncio.run(get_packages_for_paths(
-            args.path, searchers=searchers, regex=args.regex))
+        packages = asyncio.run(
+            get_packages_for_paths(
+                args.path, searchers=searchers, regex=args.regex
+            )
+        )
         for package in packages:
             print(package)
 
