@@ -19,76 +19,72 @@ __all__ = [
     "build_incrementally",
 ]
 
-from functools import partial
 import logging
 import os
 import shutil
 import sys
 import time
-from typing import List, Set, Optional, Type, Tuple
+from functools import partial
+from typing import Optional
 
+from breezy.commit import NullCommitReporter, PointlessCommit
+from breezy.plugins.debian.changelog import debcommit
+from breezy.tree import Tree
+from breezy.workingtree import WorkingTree
+from breezy.workspace import reset_tree
+from buildlog_consultant import Problem
+from buildlog_consultant.apt import (
+    AptFetchFailure,
+)
+from buildlog_consultant.common import (
+    MissingAutomakeInput,
+    MissingConfigStatusInput,
+    MissingConfigure,
+    MissingPerlFile,
+    NeedPgBuildExtUpdateControl,
+)
+from buildlog_consultant.sbuild import (
+    DebcargoUnacceptableComparator,
+    DebcargoUnacceptablePredicate,
+)
 from debian.deb822 import (
     Deb822,
     PkgRelation,
 )
-
-from breezy.commit import PointlessCommit, NullCommitReporter
-from breezy.tree import Tree
-from breezy.workingtree import WorkingTree
-
+from debmutate._rules import (
+    dh_invoke_add_with,
+    update_rules,
+)
 from debmutate.changelog import ChangelogEditor
 from debmutate.control import (
-    ensure_relation,
     ControlEditor,
-)
-from debmutate.debhelper import (
-    get_debhelper_compat_level,
+    ensure_relation,
 )
 from debmutate.deb822 import (
     Deb822Editor,
 )
+from debmutate.debhelper import (
+    get_debhelper_compat_level,
+)
 from debmutate.reformatting import (
     FormattingUnpreservable,
     GeneratedFile,
 )
 
-from breezy.workspace import reset_tree
-
-from debmutate._rules import (
-    dh_invoke_add_with,
-    update_rules,
-)
-
-from breezy.plugins.debian.changelog import debcommit
-from buildlog_consultant import Problem
-from buildlog_consultant.apt import (
-    AptFetchFailure,
-)
-from buildlog_consultant.common import (
-    MissingConfigStatusInput,
-    MissingAutomakeInput,
-    MissingConfigure,
-    NeedPgBuildExtUpdateControl,
-    MissingPerlFile,
-)
-from buildlog_consultant.sbuild import (
-    DebcargoUnacceptablePredicate,
-    DebcargoUnacceptableComparator,
-    )
-
-from .build import (
-    DetailedDebianBuildFailure,
-    UnidentifiedDebianBuildError,
-    )
-from ..logs import rotate_logfile
 from ..buildlog import problem_to_upstream_requirement
 from ..fix_build import BuildFixer, resolve_error
+from ..logs import rotate_logfile
 from ..resolver.apt import (
     AptRequirement,
 )
 from .apt import AptManager
-from .build import attempt_build, DEFAULT_BUILDER, BUILD_LOG_FILENAME
-
+from .build import (
+    BUILD_LOG_FILENAME,
+    DEFAULT_BUILDER,
+    DetailedDebianBuildFailure,
+    UnidentifiedDebianBuildError,
+    attempt_build,
+)
 
 DEFAULT_MAX_ITERATIONS = 10
 
@@ -96,14 +92,14 @@ DEFAULT_MAX_ITERATIONS = 10
 class CircularDependency(Exception):
     """Adding dependency would introduce cycle."""
 
-    def __init__(self, package):
+    def __init__(self, package) -> None:
         self.package = package
 
 
 class DebianPackagingContext:
     def __init__(
         self, tree, subpath, committer, update_changelog, commit_reporter=None
-    ):
+    ) -> None:
         self.tree = tree
         self.subpath = subpath
         self.committer = committer
@@ -114,8 +110,8 @@ class DebianPackagingContext:
         return self.tree.abspath(os.path.join(self.subpath, *parts))
 
     def commit(
-            self, summary: str,
-            update_changelog: Optional[bool] = None) -> bool:
+        self, summary: str, update_changelog: Optional[bool] = None
+    ) -> bool:
         if update_changelog is None:
             update_changelog = self.update_changelog
         with self.tree.lock_write():
@@ -125,9 +121,11 @@ class DebianPackagingContext:
                     with ChangelogEditor(cl_path) as editor:
                         editor.add_entry([summary])
                     debcommit(
-                        self.tree, committer=self.committer,
+                        self.tree,
+                        committer=self.committer,
                         subpath=self.subpath,
-                        reporter=self.commit_reporter)
+                        reporter=self.commit_reporter,
+                    )
                 else:
                     self.tree.commit(
                         message=summary,
@@ -142,15 +140,15 @@ class DebianPackagingContext:
 
 
 class PackageDependencyFixer(BuildFixer):
-    def __init__(self, context, apt_resolver):
+    def __init__(self, context, apt_resolver) -> None:
         self.apt_resolver = apt_resolver
         self.context = context
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.apt_resolver)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.apt_resolver!r})"
 
-    def __str__(self):
-        return "upstream requirement fixer(%s)" % self.apt_resolver
+    def __str__(self) -> str:
+        return f"upstream requirement fixer({self.apt_resolver})"
 
     def can_fix(self, error):
         req = problem_to_upstream_requirement(error)
@@ -195,10 +193,12 @@ def add_build_dependency(context, requirement: AptRequirement):
     if not isinstance(requirement, AptRequirement):
         raise TypeError(requirement)
 
-    if os.path.exists(context.abspath('debian/debcargo.toml')):
+    if os.path.exists(context.abspath("debian/debcargo.toml")):
         from debmutate.debcargo import DebcargoControlShimEditor
+
         control = DebcargoControlShimEditor.from_debian_dir(
-            context.abspath('debian'))
+            context.abspath("debian")
+        )
     else:
         control_path = context.abspath("debian/control")
         control = ControlEditor(path=control_path)
@@ -211,22 +211,24 @@ def add_build_dependency(context, requirement: AptRequirement):
             for rel in requirement.relations:
                 updater.source["Build-Depends"] = ensure_relation(
                     updater.source.get("Build-Depends", ""),
-                    PkgRelation.str([rel])
+                    PkgRelation.str([rel]),
                 )
     except FormattingUnpreservable as e:
         logging.info(
-            "Unable to edit %s in a way that preserves formatting.", e.path)
+            "Unable to edit %s in a way that preserves formatting.", e.path
+        )
         return False
 
     desc = requirement.pkg_relation_str()
 
     if not updater.changed:
         logging.info(
-            "Giving up; build dependency %s was already present.", desc)
+            "Giving up; build dependency %s was already present.", desc
+        )
         return False
 
     logging.info("Adding build dependency: %s", desc)
-    return context.commit("Add missing build dependency on %s." % desc)
+    return context.commit(f"Add missing build dependency on {desc}.")
 
 
 def add_test_dependency(context, testname, requirement):
@@ -255,7 +257,8 @@ def add_test_dependency(context, testname, requirement):
                     )
     except FormattingUnpreservable as e:
         logging.info(
-            "Unable to edit %s in a way that preserves formatting.", e.path)
+            "Unable to edit %s in a way that preserves formatting.", e.path
+        )
         return False
 
     desc = requirement.pkg_relation_str()
@@ -263,21 +266,24 @@ def add_test_dependency(context, testname, requirement):
     if not updater.changed:
         logging.info(
             "Giving up; dependency %s for test %s was already present.",
-            desc, testname)
+            desc,
+            testname,
+        )
         return False
 
     logging.info("Adding dependency to test %s: %s", testname, desc)
     return context.commit(
-        "Add missing dependency for test %s on %s." % (testname, desc),
+        f"Add missing dependency for test {testname} on {desc}.",
     )
 
 
-def targeted_python_versions(tree: Tree, subpath: str) -> List[str]:
+def targeted_python_versions(tree: Tree, subpath: str) -> list[str]:
     with tree.get_file(os.path.join(subpath, "debian/control")) as f:
         control = Deb822(f)
     build_depends = PkgRelation.parse_relations(
-        control.get("Build-Depends", ""))
-    all_build_deps: Set[str] = set()
+        control.get("Build-Depends", "")
+    )
+    all_build_deps: set[str] = set()
     for or_deps in build_depends:
         all_build_deps.update(or_dep["name"] for or_dep in or_deps)
     targeted = []
@@ -298,9 +304,9 @@ def python_tie_breaker(tree, subpath, reqs):
     def same(pkg, python_version):
         if pkg.startswith(python_version + "-"):
             return True
-        if pkg.startswith("lib%s-" % python_version):
+        if pkg.startswith(f"lib{python_version}-"):
             return True
-        return pkg == r'lib%s-dev' % python_version
+        return pkg == rf"lib{python_version}-dev"
 
     for python_version in targeted:
         for req in reqs:
@@ -324,7 +330,8 @@ def enable_dh_autoreconf(context, phase):
     # Debhelper >= 10 depends on dh-autoreconf and enables autoreconf by
     # default.
     debhelper_compat_version = get_debhelper_compat_level(
-        context.tree.abspath("."))
+        context.tree.abspath(".")
+    )
     if debhelper_compat_version is not None and debhelper_compat_version < 10:
 
         def add_with_autoreconf(line, target):
@@ -343,8 +350,9 @@ def enable_dh_autoreconf(context, phase):
 
 
 def fix_missing_configure(error, phase, context):
-    if (not context.tree.has_filename("configure.ac")
-            and not context.tree.has_filename("configure.in")):
+    if not context.tree.has_filename(
+        "configure.ac"
+    ) and not context.tree.has_filename("configure.in"):
         return False
 
     return enable_dh_autoreconf(context, phase)
@@ -380,7 +388,7 @@ def fix_missing_config_status_input(error, phase, context):
 
 
 class PgBuildExtOutOfDateControlFixer(BuildFixer):
-    def __init__(self, packaging_context, session, apt):
+    def __init__(self, packaging_context, session, apt) -> None:
         self.session = session
         self.context = packaging_context
         self.apt = apt
@@ -388,20 +396,22 @@ class PgBuildExtOutOfDateControlFixer(BuildFixer):
     def can_fix(self, problem):
         return isinstance(problem, NeedPgBuildExtUpdateControl)
 
-    def __repr__(self):
-        return "%s()" % (type(self).__name__,)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}()"
 
     def _fix(self, error, phase):
         logging.info("Running 'pg_buildext updatecontrol'")
-        self.apt.install(['postgresql-common'])
+        self.apt.install(["postgresql-common"])
         external_dir, internal_dir = self.session.setup_from_vcs(
-            self.context.tree, include_controldir=None,
-            subdir=self.context.subpath)
+            self.context.tree,
+            include_controldir=None,
+            subdir=self.context.subpath,
+        )
         self.session.chdir(internal_dir)
         self.session.check_call(["pg_buildext", "updatecontrol"])
         shutil.copy(
             os.path.join(external_dir, error.generated_path),
-            self.context.abspath(error.generated_path)
+            self.context.abspath(error.generated_path),
         )
         return self.context.commit(
             "Run 'pgbuildext updatecontrol'.", update_changelog=False
@@ -421,23 +431,22 @@ def fix_missing_makefile_pl(error, phase, context):
 
 def debcargo_coerce_unacceptable_prerelease(error, phase, context):
     from debmutate.debcargo import DebcargoEditor
-    with DebcargoEditor(context.abspath('debian/debcargo.toml')) as editor:
-        editor['allow_prerelease_deps'] = True
-    return context.commit('Enable allow_prerelease_deps.')
+
+    with DebcargoEditor(context.abspath("debian/debcargo.toml")) as editor:
+        editor["allow_prerelease_deps"] = True
+    return context.commit("Enable allow_prerelease_deps.")
 
 
 class SimpleBuildFixer(BuildFixer):
-    def __init__(self, packaging_context, problem_cls: Type[Problem], fn):
+    def __init__(
+        self, packaging_context, problem_cls: type[Problem], fn
+    ) -> None:
         self.context = packaging_context
         self._problem_cls = problem_cls
         self._fn = fn
 
-    def __repr__(self):
-        return "%s(%s, %s)" % (
-            type(self).__name__,
-            self._problem_cls.__name__,
-            self._fn.__name__,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self._problem_cls.__name__}, {self._fn.__name__})"
 
     def can_fix(self, problem: Problem):
         return isinstance(problem, self._problem_cls)
@@ -447,19 +456,16 @@ class SimpleBuildFixer(BuildFixer):
 
 
 class DependencyBuildFixer(BuildFixer):
-    def __init__(self, packaging_context, apt_resolver,
-                 problem_cls: Type[Problem], fn):
+    def __init__(
+        self, packaging_context, apt_resolver, problem_cls: type[Problem], fn
+    ) -> None:
         self.context = packaging_context
         self.apt_resolver = apt_resolver
         self._problem_cls = problem_cls
         self._fn = fn
 
-    def __repr__(self):
-        return "%s(%s, %s)" % (
-            type(self).__name__,
-            self._problem_cls.__name__,
-            self._fn.__name__,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self._problem_cls.__name__}, {self._fn.__name__})"
 
     def can_fix(self, problem: Problem):
         return isinstance(problem, self._problem_cls)
@@ -472,40 +478,52 @@ def versioned_package_fixers(session, packaging_context, apt: AptManager):
     return [
         PgBuildExtOutOfDateControlFixer(packaging_context, session, apt),
         SimpleBuildFixer(
-            packaging_context, MissingConfigure, fix_missing_configure),
+            packaging_context, MissingConfigure, fix_missing_configure
+        ),
         SimpleBuildFixer(
             packaging_context, MissingAutomakeInput, fix_missing_automake_input
         ),
         SimpleBuildFixer(
-            packaging_context, MissingConfigStatusInput,
-            fix_missing_config_status_input
+            packaging_context,
+            MissingConfigStatusInput,
+            fix_missing_config_status_input,
         ),
         SimpleBuildFixer(
-            packaging_context, MissingPerlFile, fix_missing_makefile_pl),
+            packaging_context, MissingPerlFile, fix_missing_makefile_pl
+        ),
         SimpleBuildFixer(
-            packaging_context, DebcargoUnacceptablePredicate,
-            debcargo_coerce_unacceptable_prerelease),
+            packaging_context,
+            DebcargoUnacceptablePredicate,
+            debcargo_coerce_unacceptable_prerelease,
+        ),
         SimpleBuildFixer(
-            packaging_context, DebcargoUnacceptableComparator,
-            debcargo_coerce_unacceptable_prerelease),
+            packaging_context,
+            DebcargoUnacceptableComparator,
+            debcargo_coerce_unacceptable_prerelease,
+        ),
     ]
 
 
-def apt_fixers(apt: AptManager, packaging_context,
-               dep_server_url: Optional[str] = None) -> List[BuildFixer]:
+def apt_fixers(
+    apt: AptManager, packaging_context, dep_server_url: Optional[str] = None
+) -> list[BuildFixer]:
     from ..resolver.apt import AptResolver
-    from .udd import popcon_tie_breaker
     from .build_deps import BuildDependencyTieBreaker
+    from .udd import popcon_tie_breaker
 
     apt_tie_breakers = [
-        partial(python_tie_breaker, packaging_context.tree,
-                packaging_context.subpath),
+        partial(
+            python_tie_breaker,
+            packaging_context.tree,
+            packaging_context.subpath,
+        ),
         BuildDependencyTieBreaker.from_session(apt.session),
         popcon_tie_breaker,
     ]
     resolver: AptResolver
     if dep_server_url:
         from ..resolver.dep_server import DepServerAptResolver
+
         resolver = DepServerAptResolver(apt, dep_server_url, apt_tie_breakers)
     else:
         resolver = AptResolver(apt, apt_tie_breakers)
@@ -518,57 +536,52 @@ def apt_fixers(apt: AptManager, packaging_context,
 
 
 def default_fixers(
-        local_tree: WorkingTree,
-        subpath: str, apt: AptManager,
-        committer: Optional[str] = None,
-        update_changelog: Optional[bool] = None,
-        dep_server_url: Optional[str] = None):
+    local_tree: WorkingTree,
+    subpath: str,
+    apt: AptManager,
+    committer: Optional[str] = None,
+    update_changelog: Optional[bool] = None,
+    dep_server_url: Optional[str] = None,
+):
     packaging_context = DebianPackagingContext(
-        local_tree, subpath, committer, update_changelog,
-        commit_reporter=NullCommitReporter()
+        local_tree,
+        subpath,
+        committer,
+        update_changelog,
+        commit_reporter=NullCommitReporter(),
     )
-    return (versioned_package_fixers(apt.session, packaging_context, apt)
-            + apt_fixers(apt, packaging_context, dep_server_url))
+    return versioned_package_fixers(
+        apt.session, packaging_context, apt
+    ) + apt_fixers(apt, packaging_context, dep_server_url)
 
 
 def build_incrementally(
     local_tree: WorkingTree,
-    apt: AptManager,
     suffix: Optional[str],
     build_suite: Optional[str],
     output_directory: str,
     build_command: str,
-    build_changelog_entry,
-    committer: Optional[str] = None,
+    fixers: list[BuildFixer],
+    build_changelog_entry: Optional[str] = None,
     max_iterations: int = DEFAULT_MAX_ITERATIONS,
     subpath: str = "",
     source_date_epoch=None,
-    update_changelog: bool = True,
     apt_repository: Optional[str] = None,
     apt_repository_key: Optional[str] = None,
-    extra_repositories: Optional[List[str]] = None,
-    fixers: Optional[List[BuildFixer]] = None,
-    run_gbp_dch: Optional[bool] = None,
-    dep_server_url: Optional[str] = None,
+    extra_repositories: Optional[list[str]] = None,
+    run_gbp_dch: bool = False,
 ):
-    fixed_errors: List[Tuple[Problem, str]] = []
-    if fixers is None:
-        fixers = default_fixers(
-            local_tree, subpath, apt, committer=committer,
-            update_changelog=update_changelog,
-            dep_server_url=dep_server_url)
+    fixed_errors: list[tuple[Problem, str]] = []
     logging.info("Using fixers: %r", fixers)
-    if run_gbp_dch is None:
-        run_gbp_dch = (update_changelog is False)
     while True:
         try:
             return attempt_build(
-                local_tree,
-                suffix,
-                build_suite,
-                output_directory,
-                build_command,
-                build_changelog_entry,
+                local_tree=local_tree,
+                suffix=suffix,
+                build_suite=build_suite,
+                output_directory=output_directory,
+                build_command=build_command,
+                build_changelog_entry=build_changelog_entry,
                 subpath=subpath,
                 source_date_epoch=source_date_epoch,
                 run_gbp_dch=run_gbp_dch,
@@ -585,18 +598,23 @@ def build_incrementally(
                 raise
             if (e.error, e.phase) in fixed_errors:
                 logging.warning(
-                    "Error was still not fixed on second try. Giving up.")
+                    "Error was still not fixed on second try. Giving up."
+                )
                 raise
-            if (max_iterations is not None
-                    and len(fixed_errors) > max_iterations):
+            if (
+                max_iterations is not None
+                and len(fixed_errors) > max_iterations
+            ):
                 logging.warning(
-                    "Last fix did not address the issue. Giving up.")
+                    "Last fix did not address the issue. Giving up."
+                )
                 raise
             reset_tree(local_tree, subpath=subpath)
             try:
                 if not resolve_error(e.error, e.phase, fixers):
                     logging.warning(
-                        "Failed to resolve error %r. Giving up.", e.error)
+                        "Failed to resolve error %r. Giving up.", e.error
+                    )
                     raise
             except GeneratedFile:
                 logging.warning(
@@ -616,21 +634,42 @@ def build_incrementally(
             rotate_logfile(os.path.join(output_directory, BUILD_LOG_FILENAME))
 
 
+def rescue_build_log(output_directory, *, tree=None):
+    xdg_cache_dir = os.environ.get(
+        "XDG_CACHE_HOME", os.path.expanduser("~/.cache")
+    )
+    buildlogs_dir = os.path.join(xdg_cache_dir, "ognibuild", "buildlogs")
+    os.makedirs(buildlogs_dir, exist_ok=True)
+    target_log_file = os.path.join(
+        buildlogs_dir,
+        "{}-{}.log".format(
+            os.path.basename(getattr(tree, "basedir", "build")),
+            time.strftime("%Y-%m-%d_%H%M%s"),
+        ),
+    )
+    shutil.copy(os.path.join(output_directory, "build.log"), target_log_file)
+    logging.info("Build log available in %s", target_log_file)
+
+
 def main(argv=None):
     import argparse
 
     parser = argparse.ArgumentParser("ognibuild.debian.fix_build")
-    modifications = parser.add_argument_group('Modifications')
+    modifications = parser.add_argument_group("Modifications")
     modifications.add_argument(
-        "--suffix", type=str, help="Suffix to use for test builds.",
-        default="fixbuild1"
+        "--suffix",
+        type=str,
+        help="Suffix to use for test builds.",
+        default="fixbuild1",
     )
     modifications.add_argument(
         "--suite", type=str, help="Suite to target.", default="unstable"
     )
     modifications.add_argument(
-        "--committer", type=str, help="Committer string (name and email)",
-        default=None
+        "--committer",
+        type=str,
+        help="Committer string (name and email)",
+        default=None,
     )
     modifications.add_argument(
         "--no-update-changelog",
@@ -646,7 +685,7 @@ def main(argv=None):
         help="force updating of the changelog",
         default=None,
     )
-    build_behaviour = parser.add_argument_group('Build Behaviour')
+    build_behaviour = parser.add_argument_group("Build Behaviour")
     build_behaviour.add_argument(
         "--output-directory", type=str, help="Output directory.", default=None
     )
@@ -658,25 +697,30 @@ def main(argv=None):
     )
 
     build_behaviour.add_argument(
-        '--max-iterations',
+        "--max-iterations",
         type=int,
         default=DEFAULT_MAX_ITERATIONS,
-        help='Maximum number of issues to attempt to fix before giving up.')
+        help="Maximum number of issues to attempt to fix before giving up.",
+    )
     build_behaviour.add_argument("--schroot", type=str, help="chroot to use.")
     parser.add_argument(
-        "--dep-server-url", type=str,
+        "--dep-server-url",
+        type=str,
         help="ognibuild dep server to use",
-        default=os.environ.get('OGNIBUILD_DEPS'))
+        default=os.environ.get("OGNIBUILD_DEPS"),
+    )
     parser.add_argument("--verbose", action="store_true", help="Be verbose")
 
     args = parser.parse_args()
-    import breezy.git  # noqa: F401
+    import contextlib
+    import tempfile
+
     import breezy.bzr  # noqa: F401
+    import breezy.git  # noqa: F401
+
     from ..session import Session
     from ..session.plain import PlainSession
     from ..session.schroot import SchrootSession
-    import tempfile
-    import contextlib
 
     if args.verbose:
         logging.basicConfig(level=logging.DEBUG, format="%(message)s")
@@ -691,13 +735,16 @@ def main(argv=None):
             output_directory = args.output_directory
             if not os.path.isdir(output_directory):
                 parser.error(
-                    'output directory %s is not a directory'
-                    % output_directory)
+                    f"output directory {output_directory} is not a directory"
+                )
 
-        tree = WorkingTree.open(".")
+        tree, subpath = WorkingTree.open_containing(".")
         session: Session
         if args.schroot:
-            session = SchrootSession(args.schroot)
+            # TODO(jelmer): pass in package name as part of session prefix
+            session = SchrootSession(
+                args.schroot, session_prefix="deb-fix-build"
+            )
         else:
             session = PlainSession()
 
@@ -705,19 +752,25 @@ def main(argv=None):
 
         apt = AptManager(session)
 
+        fixers = default_fixers(
+            tree,
+            subpath,
+            apt,
+            committer=args.committer,
+            update_changelog=args.update_changelog,
+            dep_server_url=args.dep_server_url,
+        )
+
         try:
             (changes_filenames, cl_entry) = build_incrementally(
-                tree,
-                apt,
-                args.suffix,
-                args.suite,
-                output_directory,
-                args.build_command,
-                None,
-                committer=args.committer,
-                update_changelog=args.update_changelog,
+                local_tree=tree,
+                suffix=args.suffix,
+                build_suite=args.suite,
+                output_directory=output_directory,
+                build_command=args.build_command,
+                fixers=fixers,
                 max_iterations=args.max_iterations,
-                dep_server_url=args.dep_server_url,
+                run_gbp_dch=(args.update_changelog is False),
             )
         except DetailedDebianBuildFailure as e:
             if e.phase is None:
@@ -725,23 +778,10 @@ def main(argv=None):
             elif len(e.phase) == 1:
                 phase = e.phase[0]
             else:
-                phase = "%s (%s)" % (e.phase[0], e.phase[1])
+                phase = f"{e.phase[0]} ({e.phase[1]})"
             logging.fatal("Error during %s: %s", phase, e.error)
             if not args.output_directory:
-                xdg_cache_dir = os.environ.get(
-                    'XDG_CACHE_HOME', os.path.expanduser('~/.cache'))
-                buildlogs_dir = os.path.join(
-                    xdg_cache_dir, 'ognibuild', 'buildlogs')
-                os.makedirs(buildlogs_dir, exist_ok=True)
-                target_log_file = os.path.join(
-                    buildlogs_dir,
-                    '%s-%s.log' % (
-                        os.path.basename(getattr(tree, 'basedir', 'build')),
-                        time.strftime('%Y-%m-%d_%H%M%s')))
-                shutil.copy(
-                    os.path.join(output_directory, 'build.log'),
-                    target_log_file)
-                logging.info('Build log available in %s', target_log_file)
+                rescue_build_log(output_directory, tree=tree)
             return 1
         except UnidentifiedDebianBuildError as e:
             if e.phase is None:
@@ -749,13 +789,17 @@ def main(argv=None):
             elif len(e.phase) == 1:
                 phase = e.phase[0]
             else:
-                phase = "%s (%s)" % (e.phase[0], e.phase[1])
+                phase = f"{e.phase[0]} ({e.phase[1]})"
             logging.fatal("Error during %s: %s", phase, e.description)
+            if not args.output_directory:
+                rescue_build_log(output_directory, tree=tree)
             return 1
 
         logging.info(
-            'Built %s - changes file at %r.',
-            cl_entry.version, changes_filenames)
+            "Built %s - changes file at %r.",
+            cl_entry.version,
+            changes_filenames,
+        )
 
 
 if __name__ == "__main__":
