@@ -15,16 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from typing import Dict, List
 from . import Requirement
 
 
 def run_info(session, buildsystems, fixers=None):
     for buildsystem in buildsystems:
-        deps: Dict[str, List[Requirement]] = {}
+        deps: dict[str, list[Requirement]] = {}
         try:
             for kind, dep in buildsystem.get_declared_dependencies(
-                    session, fixers=fixers):
+                session, fixers=fixers
+            ):
                 deps.setdefault(kind, []).append(dep)
         except NotImplementedError:
             print(
@@ -34,18 +34,21 @@ def run_info(session, buildsystems, fixers=None):
         if deps:
             print("\tDeclared dependencies:")
             for kind in deps:
-                print("\t\t%s:" % kind)
+                print(f"\t\t{kind}:")
                 for dep in deps[kind]:
-                    print("\t\t\t%s" % dep)
+                    print(f"\t\t\t{dep}")
             print("")
         try:
-            outputs = list(buildsystem.get_declared_outputs(
-                session, fixers=fixers))
+            outputs = list(
+                buildsystem.get_declared_outputs(session, fixers=fixers)
+            )
         except NotImplementedError:
-            print("\tUnable to detect declared outputs for this type of "
-                  "build system")
+            print(
+                "\tUnable to detect declared outputs for this type of "
+                "build system"
+            )
             outputs = []
         if outputs:
             print("\tDeclared outputs:")
             for output in outputs:
-                print("\t\t%s" % output)
+                print(f"\t\t{output}")
