@@ -15,21 +15,43 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from .buildsystem import NoBuildToolsFound, InstallTarget
+from functools import partial
 from typing import Optional
 
+from .buildsystem import InstallTarget, NoBuildToolsFound
+from .fix_build import iterate_with_build_fixers
+from .logs import NoLogManager
 
-def run_install(session, buildsystems, resolver, fixers, user: bool = False, prefix: Optional[str] = None):
+
+def run_install(
+    session,
+    buildsystems,
+    resolver,
+    fixers,
+    *,
+    user: bool = False,
+    prefix: Optional[str] = None,
+    log_manager=None,
+):
     # Some things want to write to the user's home directory,
     # e.g. pip caches in ~/.cache
     session.create_home()
 
+    if log_manager is None:
+        log_manager = NoLogManager()
+
     install_target = InstallTarget()
     install_target.user = user
     install_target.prefix = prefix
 
     for buildsystem in buildsystems:
-        buildsystem.install(session, resolver, fixers, install_target)
+        iterate_with_build_fixers(
+            fixers,
+            ["install"],
+            log_manager.wrap(
+                partial(buildsystem.install, session, resolver, install_target)
+            ),
+        )
         return
 
     raise NoBuildToolsFound()
