@@ -20,41 +20,37 @@ from . import UpstreamOutput
 
 
 class BinaryOutput(UpstreamOutput):
-    def __init__(self, name):
-        super(BinaryOutput, self).__init__("binary")
+    def __init__(self, name) -> None:
+        super().__init__("binary")
         self.name = name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.name)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.name!r})"
 
-    def __str__(self):
-        return "binary: %s" % self.name
+    def __str__(self) -> str:
+        return f"binary: {self.name}"
 
 
 class PythonPackageOutput(UpstreamOutput):
-    def __init__(self, name, python_version=None):
-        super(PythonPackageOutput, self).__init__("python-package")
+    def __init__(self, name, python_version=None) -> None:
+        super().__init__("python-package")
         self.name = name
         self.python_version = python_version
 
-    def __str__(self):
-        return "python package: %s" % self.name
+    def __str__(self) -> str:
+        return f"python package: {self.name}"
 
-    def __repr__(self):
-        return "%s(%r, python_version=%r)" % (
-            type(self).__name__,
-            self.name,
-            self.python_version,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.name!r}, python_version={self.python_version!r})"
 
 
 class RPackageOutput(UpstreamOutput):
-    def __init__(self, name):
-        super(RPackageOutput, self).__init__("r-package")
+    def __init__(self, name) -> None:
+        super().__init__("r-package")
         self.name = name
 
-    def __str__(self):
-        return "R package: %s" % self.name
+    def __str__(self) -> str:
+        return f"R package: {self.name}"
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.name)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.name!r})"
