@@ -19,20 +19,19 @@
 import posixpath
 import re
 import subprocess
-from typing import Optional, List, Set
+from typing import Optional
 
 from . import Requirement
 
 
 class PythonPackageRequirement(Requirement):
-
     family = "python-package"
 
     package: str
 
     def __init__(
-            self, package, python_version=None, specs=None,
-            minimum_version=None):
+        self, package, python_version=None, specs=None, minimum_version=None
+    ) -> None:
         self.package = package
         self.python_version = python_version
         if specs is None:
@@ -41,19 +40,14 @@ class PythonPackageRequirement(Requirement):
             specs.append((">=", minimum_version))
         self.specs = specs
 
-    def __repr__(self):
-        return "%s(%r, python_version=%r, specs=%r)" % (
-            type(self).__name__,
-            self.package,
-            self.python_version,
-            self.specs,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r}, python_version={self.python_version!r}, specs={self.specs!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.specs:
-            return "python package: %s (%r)" % (self.package, self.specs)
+            return f"python package: {self.package} ({self.specs!r})"
         else:
-            return "python package: %s" % (self.package,)
+            return f"python package: {self.package}"
 
     @classmethod
     def from_requirement_str(cls, text, python_version=None):
@@ -61,12 +55,14 @@ class PythonPackageRequirement(Requirement):
 
         req = RequirementEntry.parse(text)
         return cls(
-            package=req.name, specs=req.specs, python_version=python_version)
+            package=req.name, specs=req.specs, python_version=python_version
+        )
 
     def requirement_str(self):
         if self.specs:
-            return '%s;%s' % (
-                self.package, ','.join([''.join(s) for s in self.specs]))
+            return "{};{}".format(
+                self.package, ",".join(["".join(s) for s in self.specs])
+            )
         return self.package
 
     @classmethod
@@ -95,8 +91,11 @@ class PythonPackageRequirement(Requirement):
             raise NotImplementedError
         text = self.package + ",".join(["".join(spec) for spec in self.specs])
         p = session.Popen(
-            [cmd, "-c",
-             "import pkg_resources; pkg_resources.require(%r)" % text],
+            [
+                cmd,
+                "-c",
+                f"import pkg_resources; pkg_resources.require({text!r})",
+            ],
             stdout=subprocess.DEVNULL,
             stderr=subprocess.DEVNULL,
         )
@@ -108,14 +107,13 @@ Requirement.register_json(PythonPackageRequirement)
 
 
 class LatexPackageRequirement(Requirement):
-
     family = "latex-package"
 
-    def __init__(self, package: str):
+    def __init__(self, package: str) -> None:
         self.package = package
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.package)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r})"
 
     def _json(self):
         return self.package
@@ -129,7 +127,6 @@ Requirement.register_json(LatexPackageRequirement)
 
 
 class PhpPackageRequirement(Requirement):
-
     family = "php-package"
 
     def __init__(
@@ -138,7 +135,7 @@ class PhpPackageRequirement(Requirement):
         channel: Optional[str] = None,
         min_version: Optional[str] = None,
         max_version: Optional[str] = None,
-    ):
+    ) -> None:
         self.package = package
         self.channel = channel
         self.min_version = min_version
@@ -151,25 +148,18 @@ class PhpPackageRequirement(Requirement):
     def _from_json(cls, js):
         return cls(*js)
 
-    def __repr__(self):
-        return "%s(%r, %r, %r, %r)" % (
-            type(self).__name__,
-            self.package,
-            self.channel,
-            self.min_version,
-            self.max_version,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r}, {self.channel!r}, {self.min_version!r}, {self.max_version!r})"
 
 
 Requirement.register_json(PhpPackageRequirement)
 
 
 class BinaryRequirement(Requirement):
-
     family = "binary"
     binary_name: str
 
-    def __init__(self, binary_name):
+    def __init__(self, binary_name) -> None:
         self.binary_name = binary_name
 
     def _json(self):
@@ -179,8 +169,8 @@ class BinaryRequirement(Requirement):
     def _from_json(cls, js):
         return cls(js)
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.binary_name)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.binary_name!r})"
 
     def met(self, session):
         p = session.Popen(
@@ -196,50 +186,46 @@ Requirement.register_json(BinaryRequirement)
 
 
 class PHPExtensionRequirement(Requirement):
-
     family = "php-extension"
     extension: str
 
-    def __init__(self, extension: str):
+    def __init__(self, extension: str) -> None:
         self.extension = extension
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.extension)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.extension!r})"
 
 
 class PytestPluginRequirement(Requirement):
-
     family = "pytest-plugin"
 
     plugin: str
 
-    def __init__(self, plugin: str):
+    def __init__(self, plugin: str) -> None:
         self.plugin = plugin
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.plugin)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.plugin!r})"
 
 
 class VcsControlDirectoryAccessRequirement(Requirement):
-
-    vcs: List[str]
+    vcs: list[str]
     family = "vcs-access"
 
-    def __init__(self, vcs):
+    def __init__(self, vcs) -> None:
         self.vcs = vcs
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.vcs)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.vcs!r})"
 
 
 class PerlModuleRequirement(Requirement):
-
     module: str
     filename: Optional[str]
-    inc: Optional[List[str]]
+    inc: Optional[list[str]]
     family = "perl-module"
 
-    def __init__(self, module, filename=None, inc=None):
+    def __init__(self, module, filename=None, inc=None) -> None:
         self.module = module
         self.filename = filename
         self.inc = inc
@@ -248,17 +234,16 @@ class PerlModuleRequirement(Requirement):
     def relfilename(self):
         return self.module.replace("::", "/") + ".pm"
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.module)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.module!r})"
 
 
 class VagueDependencyRequirement(Requirement):
-
     name: str
     family = "vague"
     minimum_version: Optional[str] = None
 
-    def __init__(self, name, minimum_version=None):
+    def __init__(self, name, minimum_version=None) -> None:
         self.name = name
         self.minimum_version = minimum_version
 
@@ -267,119 +252,119 @@ class VagueDependencyRequirement(Requirement):
             yield BinaryRequirement(self.name)
             yield LibraryRequirement(self.name)
             yield PkgConfigRequirement(
-                self.name, minimum_version=self.minimum_version)
+                self.name, minimum_version=self.minimum_version
+            )
             if self.name.lower() != self.name:
                 yield BinaryRequirement(self.name.lower())
                 yield LibraryRequirement(self.name.lower())
                 yield PkgConfigRequirement(
-                    self.name.lower(), minimum_version=self.minimum_version)
+                    self.name.lower(), minimum_version=self.minimum_version
+                )
             try:
                 from .resolver.apt import AptRequirement
             except ModuleNotFoundError:
                 pass
             else:
                 yield AptRequirement.simple(
-                    self.name.lower(), minimum_version=self.minimum_version)
-                if self.name.lower().startswith('lib'):
-                    devname = '%s-dev' % self.name.lower()
+                    self.name.lower(), minimum_version=self.minimum_version
+                )
+                if self.name.lower().startswith("lib"):
+                    devname = f"{self.name.lower()}-dev"
                 else:
-                    devname = 'lib%s-dev' % self.name.lower()
+                    devname = f"lib{self.name.lower()}-dev"
                 yield AptRequirement.simple(
-                    devname, minimum_version=self.minimum_version)
+                    devname, minimum_version=self.minimum_version
+                )
 
     def met(self, session):
         return any(x.met(session) for x in self.expand())
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.name)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.name!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.minimum_version:
-            return "%s >= %s" % (self.name, self.minimum_version)
+            return f"{self.name} >= {self.minimum_version}"
         return self.name
 
 
 class NodePackageRequirement(Requirement):
-
     package: str
     family = "npm-package"
 
-    def __init__(self, package):
+    def __init__(self, package) -> None:
         self.package = package
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.package)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r})"
 
 
 class LuaModuleRequirement(Requirement):
-
     module: str
     family = "lua-module"
 
-    def __init__(self, module):
+    def __init__(self, module) -> None:
         self.module = module
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.module)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.module!r})"
 
 
 class PerlPreDeclaredRequirement(Requirement):
-
     name: str
 
     # TODO(jelmer): Can we obtain this information elsewhere?
     KNOWN_MODULES = {
-        'auto_set_repository': 'Module::Install::Repository',
-        'author_tests': 'Module::Install::AuthorTests',
-        'recursive_author_tests': 'Module::Install::AuthorTests',
-        'author_requires': 'Module::Install::AuthorRequires',
-        'readme_from': 'Module::Install::ReadmeFromPod',
-        'catalyst': 'Module::Install::Catalyst',
-        'githubmeta': 'Module::Install::GithubMeta',
-        'use_ppport': 'Module::Install::XSUtil',
-        'pod_from': 'Module::Install::PodFromEuclid',
-        'write_doap_changes': 'Module::Install::DOAPChangeSets',
-        'use_test_base': 'Module::Install::TestBase',
-        'jsonmeta': 'Module::Install::JSONMETA',
-        'extra_tests': 'Module::Install::ExtraTests',
-        'auto_set_bugtracker': 'Module::Install::Bugtracker',
-        }
+        "auto_set_repository": "Module::Install::Repository",
+        "author_tests": "Module::Install::AuthorTests",
+        "recursive_author_tests": "Module::Install::AuthorTests",
+        "author_requires": "Module::Install::AuthorRequires",
+        "readme_from": "Module::Install::ReadmeFromPod",
+        "catalyst": "Module::Install::Catalyst",
+        "githubmeta": "Module::Install::GithubMeta",
+        "use_ppport": "Module::Install::XSUtil",
+        "pod_from": "Module::Install::PodFromEuclid",
+        "write_doap_changes": "Module::Install::DOAPChangeSets",
+        "use_test_base": "Module::Install::TestBase",
+        "jsonmeta": "Module::Install::JSONMETA",
+        "extra_tests": "Module::Install::ExtraTests",
+        "auto_set_bugtracker": "Module::Install::Bugtracker",
+    }
 
     family = "perl-predeclared"
 
-    def __init__(self, name):
+    def __init__(self, name) -> None:
         self.name = name
 
     def lookup_module(self):
         module = self.KNOWN_MODULES[self.name]
         return PerlModuleRequirement(module=module)
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.name)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.name!r})"
 
 
 class NodeModuleRequirement(Requirement):
-
     module: str
     family = "npm-module"
 
-    def __init__(self, module):
+    def __init__(self, module) -> None:
         self.module = module
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.module)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.module!r})"
 
 
 class CargoCrateRequirement(Requirement):
-
     crate: str
-    features: Set[str]
+    features: set[str]
     api_version: Optional[str]
     minimum_version: Optional[str]
     family = "cargo-crate"
 
-    def __init__(self, crate, features=None, api_version=None,
-                 minimum_version=None):
+    def __init__(
+        self, crate, features=None, api_version=None, minimum_version=None
+    ) -> None:
         self.crate = crate
         if features is None:
             features = set()
@@ -387,62 +372,50 @@ class CargoCrateRequirement(Requirement):
         self.api_version = api_version
         self.minimum_version = minimum_version
 
-    def __repr__(self):
-        return "%s(%r, features=%r, api_version=%r, minimum_version=%r)" % (
-            type(self).__name__,
-            self.crate,
-            self.features,
-            self.api_version,
-            self.minimum_version,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.crate!r}, features={self.features!r}, api_version={self.api_version!r}, minimum_version={self.minimum_version!r})"
 
-    def __str__(self):
-        ret = "cargo crate: %s %s" % (
-            self.crate,
-            self.api_version or "")
+    def __str__(self) -> str:
+        ret = "cargo crate: {} {}".format(self.crate, self.api_version or "")
         if self.features:
-            ret += " (%s)" % (", ".join(sorted(self.features)))
+            ret += " ({})".format(", ".join(sorted(self.features)))
         if self.minimum_version:
-            ret += " (>= %s)" % self.minimum_version
+            ret += f" (>= {self.minimum_version})"
         return ret
 
 
 class PkgConfigRequirement(Requirement):
-
     module: str
     family = "pkg-config"
 
-    def __init__(self, module, minimum_version=None):
+    def __init__(self, module, minimum_version=None) -> None:
         self.module = module
         self.minimum_version = minimum_version
 
-    def __repr__(self):
-        return "%s(%r, minimum_version=%r)" % (
-            type(self).__name__, self.module, self.minimum_version)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.module!r}, minimum_version={self.minimum_version!r})"
 
 
 class PathRequirement(Requirement):
-
     path: str
     family = "path"
 
-    def __init__(self, path):
+    def __init__(self, path) -> None:
         self.path = path
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.path)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.path!r})"
 
 
 class CHeaderRequirement(Requirement):
-
     header: str
     family = "c-header"
 
-    def __init__(self, header):
+    def __init__(self, header) -> None:
         self.header = header
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.header)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.header!r})"
 
 
 class JavaScriptRuntimeRequirement(Requirement):
@@ -450,98 +423,87 @@ class JavaScriptRuntimeRequirement(Requirement):
 
 
 class ValaPackageRequirement(Requirement):
-
     package: str
     family = "vala"
 
-    def __init__(self, package: str):
+    def __init__(self, package: str) -> None:
         self.package = package
 
 
 class RubyGemRequirement(Requirement):
-
     gem: str
     minimum_version: Optional[str]
     family = "gem"
 
-    def __init__(self, gem: str, minimum_version: Optional[str]):
+    def __init__(self, gem: str, minimum_version: Optional[str]) -> None:
         self.gem = gem
         self.minimum_version = minimum_version
 
 
 class GoPackageRequirement(Requirement):
-
     package: str
     version: Optional[str]
     family = "go-package"
 
-    def __init__(self, package: str, version: Optional[str] = None):
+    def __init__(self, package: str, version: Optional[str] = None) -> None:
         self.package = package
         self.version = version
 
-    def __repr__(self):
-        return "%s(%r, version=%r)" % (
-            type(self).__name__, self.package, self.version)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r}, version={self.version!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.version:
-            return "go package: %s (= %s)" % (self.package, self.version)
-        return "go package: %s" % self.package
+            return f"go package: {self.package} (= {self.version})"
+        return f"go package: {self.package}"
 
 
 class GoRequirement(Requirement):
-
     version: Optional[str]
     family = "go"
 
-    def __init__(self, version: Optional[str] = None):
+    def __init__(self, version: Optional[str] = None) -> None:
         self.version = version
 
-    def __str__(self):
-        return "go %s" % self.version
+    def __str__(self) -> str:
+        return f"go {self.version}"
 
 
 class DhAddonRequirement(Requirement):
-
     path: str
     family = "dh-addon"
 
-    def __init__(self, path: str):
+    def __init__(self, path: str) -> None:
         self.path = path
 
 
 class PhpClassRequirement(Requirement):
-
     php_class: str
     family = "php-class"
 
-    def __init__(self, php_class: str):
+    def __init__(self, php_class: str) -> None:
         self.php_class = php_class
 
 
 class RPackageRequirement(Requirement):
-
     package: str
     minimum_version: Optional[str]
     family = "r-package"
 
-    def __init__(self, package: str, minimum_version: Optional[str] = None):
+    def __init__(
+        self, package: str, minimum_version: Optional[str] = None
+    ) -> None:
         self.package = package
         self.minimum_version = minimum_version
 
-    def __repr__(self):
-        return "%s(%r, minimum_version=%r)" % (
-            type(self).__name__,
-            self.package,
-            self.minimum_version,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r}, minimum_version={self.minimum_version!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.minimum_version:
-            return "R package: %s (>= %s)" % (
-                self.package, self.minimum_version)
+            return f"R package: {self.package} (>= {self.minimum_version})"
         else:
-            return "R package: %s" % (self.package,)
+            return f"R package: {self.package}"
 
     @classmethod
     def from_str(cls, text):
@@ -556,28 +518,24 @@ class RPackageRequirement(Requirement):
 
 
 class OctavePackageRequirement(Requirement):
-
     package: str
     minimum_version: Optional[str]
     family = "octave-package"
 
-    def __init__(self, package: str, minimum_version: Optional[str] = None):
+    def __init__(
+        self, package: str, minimum_version: Optional[str] = None
+    ) -> None:
         self.package = package
         self.minimum_version = minimum_version
 
-    def __repr__(self):
-        return "%s(%r, minimum_version=%r)" % (
-            type(self).__name__,
-            self.package,
-            self.minimum_version,
-        )
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.package!r}, minimum_version={self.minimum_version!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.minimum_version:
-            return "Octave package: %s (>= %s)" % (
-                self.package, self.minimum_version)
+            return f"Octave package: {self.package} (>= {self.minimum_version})"
         else:
-            return "Octave package: %s" % (self.package,)
+            return f"Octave package: {self.package}"
 
     @classmethod
     def from_str(cls, text):
@@ -592,86 +550,78 @@ class OctavePackageRequirement(Requirement):
 
 
 class LibraryRequirement(Requirement):
-
     library: str
     family = "lib"
 
-    def __init__(self, library: str):
+    def __init__(self, library: str) -> None:
         self.library = library
 
-    def __str__(self):
-        return "Library: %s" % self.library
+    def __str__(self) -> str:
+        return f"Library: {self.library}"
 
 
 class StaticLibraryRequirement(Requirement):
-
     library: str
     filename: str
     family = "static-lib"
 
-    def __init__(self, library: str, filename: str):
+    def __init__(self, library: str, filename: str) -> None:
         self.library = library
         self.filename = filename
 
-    def __str__(self):
-        return "Static Library: %s" % self.library
+    def __str__(self) -> str:
+        return f"Static Library: {self.library}"
 
 
 class RubyFileRequirement(Requirement):
-
     filename: str
     family = "ruby-file"
 
-    def __init__(self, filename: str):
+    def __init__(self, filename: str) -> None:
         self.filename = filename
 
 
 class XmlEntityRequirement(Requirement):
-
     url: str
     family = "xml-entity"
 
-    def __init__(self, url: str):
+    def __init__(self, url: str) -> None:
         self.url = url
 
 
 class SprocketsFileRequirement(Requirement):
-
     content_type: str
     name: str
     family = "sprockets-file"
 
-    def __init__(self, content_type: str, name: str):
+    def __init__(self, content_type: str, name: str) -> None:
         self.content_type = content_type
         self.name = name
 
 
 class JavaClassRequirement(Requirement):
-
     classname: str
     family = "java-class"
 
-    def __init__(self, classname: str):
+    def __init__(self, classname: str) -> None:
         self.classname = classname
 
 
 class CMakefileRequirement(Requirement):
-
     filename: str
     version: Optional[str]
     family = "cmake-file"
 
-    def __init__(self, filename: str, version=None):
+    def __init__(self, filename: str, version=None) -> None:
         self.filename = filename
         self.version = version
 
 
 class HaskellPackageRequirement(Requirement):
-
     package: str
     family = "haskell-package"
 
-    def __init__(self, package: str, specs=None):
+    def __init__(self, package: str, specs=None) -> None:
         self.package = package
         self.specs = specs
 
@@ -682,30 +632,23 @@ class HaskellPackageRequirement(Requirement):
 
 
 class MavenArtifactRequirement(Requirement):
-
     group_id: str
     artifact_id: str
     version: Optional[str]
     kind: Optional[str]
     family = "maven-artifact"
 
-    def __init__(self, group_id, artifact_id, version=None, kind=None):
+    def __init__(self, group_id, artifact_id, version=None, kind=None) -> None:
         self.group_id = group_id
         self.artifact_id = artifact_id
         self.version = version
         self.kind = kind
 
-    def __str__(self):
-        return "maven requirement: %s:%s:%s" % (
-            self.group_id,
-            self.artifact_id,
-            self.version,
-        )
+    def __str__(self) -> str:
+        return f"maven requirement: {self.group_id}:{self.artifact_id}:{self.version}"
 
-    def __repr__(self):
-        return "%s(group_id=%r, artifact_id=%r, version=%r, kind=%r)" % (
-            type(self).__name__, self.group_id, self.artifact_id,
-            self.version, self.kind)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}(group_id={self.group_id!r}, artifact_id={self.artifact_id!r}, version={self.version!r}, kind={self.kind!r})"
 
     @classmethod
     def from_str(cls, text):
@@ -723,7 +666,7 @@ class MavenArtifactRequirement(Requirement):
             (group_id, artifact_id) = parts
             kind = "jar"
         else:
-            raise ValueError("invalid number of parts to artifact %r" % parts)
+            raise ValueError(f"invalid number of parts to artifact {parts!r}")
         return cls(group_id, artifact_id, version, kind)
 
 
@@ -732,12 +675,11 @@ class GnomeCommonRequirement(Requirement):
 
 
 class JDKFileRequirement(Requirement):
-
     jdk_path: str
     filename: str
     family = "jdk-file"
 
-    def __init__(self, jdk_path: str, filename: str):
+    def __init__(self, jdk_path: str, filename: str) -> None:
         self.jdk_path = jdk_path
         self.filename = filename
 
@@ -757,7 +699,7 @@ class JRERequirement(Requirement):
 class QtModuleRequirement(Requirement):
     family = "qt-module"
 
-    def __init__(self, module):
+    def __init__(self, module) -> None:
         self.module = module
 
 
@@ -772,25 +714,23 @@ class X11Requirement(Requirement):
 class CertificateAuthorityRequirement(Requirement):
     family = "ca-cert"
 
-    def __init__(self, url):
+    def __init__(self, url) -> None:
         self.url = url
 
 
 class PerlFileRequirement(Requirement):
-
     filename: str
     family = "perl-file"
 
-    def __init__(self, filename: str):
+    def __init__(self, filename: str) -> None:
         self.filename = filename
 
 
 class AutoconfMacroRequirement(Requirement):
-
     family = "autoconf-macro"
     macro: str
 
-    def __init__(self, macro: str):
+    def __init__(self, macro: str) -> None:
         self.macro = macro
 
     def _json(self):
@@ -811,18 +751,19 @@ class LibtoolRequirement(Requirement):
 class IntrospectionTypelibRequirement(Requirement):
     family = "introspection-type-lib"
 
-    def __init__(self, library):
+    def __init__(self, library) -> None:
         self.library = library
 
 
 class PythonModuleRequirement(Requirement):
-
     module: str
     python_version: Optional[str]
     minimum_version: Optional[str]
     family = "python-module"
 
-    def __init__(self, module, python_version=None, minimum_version=None):
+    def __init__(
+        self, module, python_version=None, minimum_version=None
+    ) -> None:
         self.module = module
         self.python_version = python_version
         self.minimum_version = minimum_version
@@ -841,41 +782,36 @@ class PythonModuleRequirement(Requirement):
         else:
             raise NotImplementedError
         p = session.Popen(
-            [cmd, "-c", "import %s" % self.module],
+            [cmd, "-c", f"import {self.module}"],
             stdout=subprocess.DEVNULL,
             stderr=subprocess.DEVNULL,
         )
         p.communicate()
         return p.returncode == 0
 
-    def __repr__(self):
-        return "%s(%r, python_version=%r, minimum_version=%r)" % (
-            type(self).__name__, self.module, self.python_version,
-            self.minimum_version)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.module!r}, python_version={self.python_version!r}, minimum_version={self.minimum_version!r})"
 
 
 class BoostComponentRequirement(Requirement):
-
     name: str
     family = "boost-component"
 
-    def __init__(self, name):
+    def __init__(self, name) -> None:
         self.name = name
 
 
 class KF5ComponentRequirement(Requirement):
-
     name: str
     family = "kf5-component"
 
-    def __init__(self, name):
+    def __init__(self, name) -> None:
         self.name = name
 
 
 class GnulibDirectoryRequirement(Requirement):
-
     directory: str
     family = "gnulib"
 
-    def __init__(self, directory):
+    def __init__(self, directory) -> None:
         self.directory = directory
