@@ -18,54 +18,54 @@
 
 import logging
 import subprocess
-from typing import Optional, List, Type, Iterator, Dict
+from collections.abc import Iterator
+from typing import Optional
 
-from .. import UnidentifiedError, Requirement
+from .. import Requirement, UnidentifiedError
 from ..fix_build import run_detecting_problems
 from ..session import Session
 
 
 class UnsatisfiedRequirements(Exception):
-    def __init__(self, reqs):
+    def __init__(self, reqs) -> None:
         self.requirements = reqs
 
 
 class Resolver:
-
     name: str
 
-    def __init__(self, session: Session, user_local: bool):
+    def __init__(self, session: Session, user_local: bool) -> None:
         raise NotImplementedError
 
-    def install(self, requirements: List[Requirement]) -> None:
+    def install(self, requirements: list[Requirement]) -> None:
         raise NotImplementedError(self.install)
 
     def resolve(self, requirement: Requirement) -> Optional[Requirement]:
         raise NotImplementedError(self.resolve)
 
-    def resolve_all(self, requirement: Requirement) -> List[Requirement]:
+    def resolve_all(self, requirement: Requirement) -> list[Requirement]:
         raise NotImplementedError(self.resolve_all)
 
-    def explain(self, requirements: List[Requirement]) -> Iterator[List[str]]:
+    def explain(self, requirements: list[Requirement]) -> Iterator[list[str]]:
         raise NotImplementedError(self.explain)
 
-    def env(self) -> Dict[str, str]:
+    def env(self) -> dict[str, str]:
         return {}
 
 
 class CPANResolver(Resolver):
     name = "cpan"
 
-    def __init__(self, session, user_local=False, skip_tests=True):
+    def __init__(self, session, user_local=False, skip_tests=True) -> None:
         self.session = session
         self.user_local = user_local
         self.skip_tests = skip_tests
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     def _cmd(self, reqs):
         ret = ["cpan", "-i"]
@@ -117,24 +117,24 @@ class CPANResolver(Resolver):
 
 
 class TlmgrResolver(Resolver):
-    def __init__(self, session, repository: str, user_local=False):
+    def __init__(self, session, repository: str, user_local=False) -> None:
         self.session = session
         self.user_local = user_local
         self.repository = repository
 
-    def __str__(self):
-        if (self.repository.startswith('http://')
-                or self.repository.startswith('https://')):
-            return 'tlmgr(%r)' % self.repository
+    def __str__(self) -> str:
+        if self.repository.startswith("http://") or self.repository.startswith(
+            "https://"
+        ):
+            return f"tlmgr({self.repository!r})"
         else:
             return self.repository
 
-    def __repr__(self):
-        return "%s(%r, %r)" % (
-            type(self).__name__, self.session, self.repository)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r}, {self.repository!r})"
 
     def _cmd(self, reqs):
-        ret = ["tlmgr", "--repository=%s" % self.repository, "install"]
+        ret = ["tlmgr", f"--repository={self.repository}", "install"]
         if self.user_local:
             ret.append("--usermode")
         ret.extend([req.package for req in reqs])
@@ -169,9 +169,11 @@ class TlmgrResolver(Resolver):
             try:
                 run_detecting_problems(self.session, cmd, user=user)
             except UnidentifiedError as e:
-                if ("tlmgr: user mode not initialized, "
-                        "please read the documentation!") in e.lines:
-                    self.session.check_call(['tlmgr', 'init-usertree'])
+                if (
+                    "tlmgr: user mode not initialized, "
+                    "please read the documentation!"
+                ) in e.lines:
+                    self.session.check_call(["tlmgr", "init-usertree"])
                 else:
                     raise
         if missing:
@@ -181,32 +183,30 @@ class TlmgrResolver(Resolver):
 class CTANResolver(TlmgrResolver):
     name = "ctan"
 
-    def __init__(self, session, user_local=False):
-        super(CTANResolver, self).__init__(
-            session, "ctan", user_local=user_local)
+    def __init__(self, session, user_local=False) -> None:
+        super().__init__(session, "ctan", user_local=user_local)
 
 
 class RResolver(Resolver):
-
     name: str
 
-    def __init__(self, session, repos, user_local=False):
+    def __init__(self, session, repos, user_local=False) -> None:
         self.session = session
         self.repos = repos
         self.user_local = user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r, %r)" % (type(self).__name__, self.session, self.repos)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r}, {self.repos!r})"
 
     def _cmd(self, req):
         # TODO(jelmer: Handle self.user_local
         return [
             "R",
             "-e",
-            "install.packages('%s', repos=%r)" % (req.package, self.repos),
+            f"install.packages('{req.package}', repos={self.repos!r})",
         ]
 
     def explain(self, requirements):
@@ -243,19 +243,19 @@ class RResolver(Resolver):
 class OctaveForgeResolver(Resolver):
     name = "octave-forge"
 
-    def __init__(self, session, user_local=False):
+    def __init__(self, session, user_local=False) -> None:
         self.session = session
         self.user_local = user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     def _cmd(self, req):
         # TODO(jelmer: Handle self.user_local
-        return ["octave-cli", "--eval", "pkg install -forge %s" % req.package]
+        return ["octave-cli", "--eval", f"pkg install -forge {req.package}"]
 
     def explain(self, requirements):
         from ..requirements import OctavePackageRequirement
@@ -291,8 +291,8 @@ class OctaveForgeResolver(Resolver):
 class CRANResolver(RResolver):
     name = "cran"
 
-    def __init__(self, session, user_local=False):
-        super(CRANResolver, self).__init__(
+    def __init__(self, session, user_local=False) -> None:
+        super().__init__(
             session, "http://cran.r-project.org", user_local=user_local
         )
 
@@ -300,33 +300,34 @@ class CRANResolver(RResolver):
 class BioconductorResolver(RResolver):
     name = "bioconductor"
 
-    def __init__(self, session, user_local=False):
-        super(BioconductorResolver, self).__init__(
-            session, "https://hedgehog.fhcrc.org/bioconductor",
-            user_local=user_local
+    def __init__(self, session, user_local=False) -> None:
+        super().__init__(
+            session,
+            "https://hedgehog.fhcrc.org/bioconductor",
+            user_local=user_local,
         )
 
 
 class HackageResolver(Resolver):
-
     name = "hackage"
 
-    def __init__(self, session, user_local=False):
+    def __init__(self, session, user_local=False) -> None:
         self.session = session
         self.user_local = user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     def _cmd(self, reqs):
         extra_args = []
         if self.user_local:
             extra_args.append("--user")
-        return (["cabal", "install"] + extra_args
-                + [req.package for req in reqs])
+        return (
+            ["cabal", "install"] + extra_args + [req.package for req in reqs]
+        )
 
     def install(self, requirements):
         from ..requirements import HaskellPackageRequirement
@@ -360,18 +361,17 @@ class HackageResolver(Resolver):
 
 
 class PypiResolver(Resolver):
-
     name = "pypi"
 
-    def __init__(self, session, user_local=False):
+    def __init__(self, session, user_local=False) -> None:
         self.session = session
         self.user_local = user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     def _cmd(self, reqs):
         extra_args = []
@@ -414,18 +414,17 @@ class PypiResolver(Resolver):
 
 
 class GoResolver(Resolver):
-
     name = "go"
 
-    def __init__(self, session, user_local):
+    def __init__(self, session, user_local) -> None:
         self.session = session
         self.user_local = user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     def install(self, requirements):
         from ..requirements import GoPackageRequirement
@@ -476,24 +475,25 @@ NPM_COMMAND_PACKAGES = {
 class NpmResolver(Resolver):
     name = "npm"
 
-    def __init__(self, session, user_local=False):
+    def __init__(self, session, user_local=False) -> None:
         self.session = session
         self.user_local = user_local
         # TODO(jelmer): Handle user_local
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.session)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.session!r})"
 
     @classmethod
     def _to_node_package_req(cls, requirement):
         from ..requirements import (
-            NodePackageRequirement,
-            NodeModuleRequirement,
             BinaryRequirement,
+            NodeModuleRequirement,
+            NodePackageRequirement,
         )
+
         if isinstance(requirement, BinaryRequirement):
             try:
                 package = NPM_COMMAND_PACKAGES[requirement.binary_name]
@@ -504,8 +504,8 @@ class NpmResolver(Resolver):
         if isinstance(requirement, NodeModuleRequirement):
             # TODO: Is this legit?
             parts = requirement.module.split("/")
-            if parts[0].startswith('@'):
-                return NodePackageRequirement('/'.join(parts[:2]))
+            if parts[0].startswith("@"):
+                return NodePackageRequirement("/".join(parts[:2]))
             else:
                 return NodePackageRequirement(parts[0])
         if isinstance(requirement, NodePackageRequirement):
@@ -526,7 +526,7 @@ class NpmResolver(Resolver):
                 continue
             cmd = ["npm", "install"]
             if not self.user_local:
-                cmd.append('-g')
+                cmd.append("-g")
             cmd.append(node_pkg_requirement.package)
             logging.info("npm: running %r", cmd)
             run_detecting_problems(self.session, cmd, user=user)
@@ -546,13 +546,13 @@ class NpmResolver(Resolver):
 
 
 class StackedResolver(Resolver):
-    def __init__(self, subs):
+    def __init__(self, subs) -> None:
         self.subs = subs
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.subs)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.subs!r})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "[" + ", ".join(map(str, self.subs)) + "]"
 
     def env(self):
@@ -578,7 +578,7 @@ class StackedResolver(Resolver):
             raise UnsatisfiedRequirements(requirements)
 
 
-NATIVE_RESOLVER_CLS: List[Type[Resolver]] = [
+NATIVE_RESOLVER_CLS: list[type[Resolver]] = [
     CPANResolver,
     CTANResolver,
     PypiResolver,
@@ -593,11 +593,13 @@ NATIVE_RESOLVER_CLS: List[Type[Resolver]] = [
 
 def native_resolvers(session, user_local):
     return StackedResolver(
-        [kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
+        [kls(session, user_local) for kls in NATIVE_RESOLVER_CLS]
+    )
 
 
-def select_resolvers(session, user_local, resolvers,
-                     dep_server_url=None) -> Optional[Resolver]:
+def select_resolvers(
+    session, user_local, resolvers, dep_server_url=None
+) -> Optional[Resolver]:
     selected = []
     for resolver in resolvers:
         for kls in NATIVE_RESOLVER_CLS:
@@ -605,19 +607,26 @@ def select_resolvers(session, user_local, resolvers,
                 selected.append(kls(session, user_local))
                 break
         else:
-            if resolver == 'native':
-                selected.extend([
-                    kls(session, user_local) for kls in NATIVE_RESOLVER_CLS])
-            elif resolver == 'apt':
+            if resolver == "native":
+                selected.extend(
+                    [kls(session, user_local) for kls in NATIVE_RESOLVER_CLS]
+                )
+            elif resolver == "apt":
                 if user_local:
                     raise NotImplementedError(
-                        'user local not supported for apt')
+                        "user local not supported for apt"
+                    )
                 if dep_server_url:
                     from .dep_server import DepServerAptResolver
-                    selected.append(DepServerAptResolver.from_session(
-                        session, dep_server_url))
+
+                    selected.append(
+                        DepServerAptResolver.from_session(
+                            session, dep_server_url
+                        )
+                    )
                 else:
                     from .apt import AptResolver
+
                     selected.append(AptResolver.from_session(session))
             else:
                 raise KeyError(resolver)
@@ -628,12 +637,15 @@ def select_resolvers(session, user_local, resolvers,
     return StackedResolver(selected)
 
 
-def auto_resolver(session: Session, explain: bool = False,
-                  system_wide: Optional[bool] = None,
-                  dep_server_url: Optional[str] = None):
+def auto_resolver(
+    session: Session,
+    explain: bool = False,
+    system_wide: Optional[bool] = None,
+    dep_server_url: Optional[str] = None,
+):
     # if session is SchrootSession or if we're root, use apt
-    from ..session.schroot import SchrootSession
     from ..session import get_user
+    from ..session.schroot import SchrootSession
 
     user = get_user(session)
     resolvers = []
@@ -652,10 +664,13 @@ def auto_resolver(session: Session, explain: bool = False,
         else:
             if dep_server_url:
                 from .dep_server import DepServerAptResolver
+
                 resolvers.append(
-                    DepServerAptResolver.from_session(session, dep_server_url))
+                    DepServerAptResolver.from_session(session, dep_server_url)
+                )
             else:
                 resolvers.append(AptResolver.from_session(session))
-    resolvers.extend([kls(session, not system_wide)
-                      for kls in NATIVE_RESOLVER_CLS])
+    resolvers.extend(
+        [kls(session, not system_wide) for kls in NATIVE_RESOLVER_CLS]
+    )
     return StackedResolver(resolvers)
