@@ -20,73 +20,73 @@ import logging
 import os
 import posixpath
 import re
-from typing import Optional, List, Tuple, Callable, Type, Awaitable
+from collections.abc import Awaitable
+from typing import Callable, Optional
 
 from debian.changelog import Version
 from debian.deb822 import PkgRelation
 
-from ..debian.apt import AptManager
-
-from . import Resolver, UnsatisfiedRequirements
 from .. import OneOfRequirement
+from ..debian.apt import AptManager
 from ..requirements import (
-    Requirement,
-    CargoCrateRequirement,
+    AutoconfMacroRequirement,
     BinaryRequirement,
+    BoostComponentRequirement,
+    CargoCrateRequirement,
+    CertificateAuthorityRequirement,
     CHeaderRequirement,
-    PkgConfigRequirement,
-    PathRequirement,
-    JavaScriptRuntimeRequirement,
-    ValaPackageRequirement,
-    RubyGemRequirement,
+    CMakefileRequirement,
+    DhAddonRequirement,
+    GnomeCommonRequirement,
     GoPackageRequirement,
     GoRequirement,
-    DhAddonRequirement,
-    PhpClassRequirement,
-    PhpPackageRequirement,
-    RPackageRequirement,
-    NodeModuleRequirement,
-    NodePackageRequirement,
-    LibraryRequirement,
-    BoostComponentRequirement,
-    KF5ComponentRequirement,
-    StaticLibraryRequirement,
-    RubyFileRequirement,
-    XmlEntityRequirement,
-    OctavePackageRequirement,
-    SprocketsFileRequirement,
-    JavaClassRequirement,
-    CMakefileRequirement,
     HaskellPackageRequirement,
-    MavenArtifactRequirement,
-    GnomeCommonRequirement,
+    IntrospectionTypelibRequirement,
+    JavaClassRequirement,
+    JavaScriptRuntimeRequirement,
     JDKFileRequirement,
     JDKRequirement,
     JRERequirement,
-    QTRequirement,
-    QtModuleRequirement,
-    X11Requirement,
-    PerlModuleRequirement,
+    KF5ComponentRequirement,
+    LibraryRequirement,
+    LibtoolRequirement,
+    MavenArtifactRequirement,
+    NodeModuleRequirement,
+    NodePackageRequirement,
+    OctavePackageRequirement,
+    PathRequirement,
     PerlFileRequirement,
-    AutoconfMacroRequirement,
+    PerlModuleRequirement,
+    PerlPreDeclaredRequirement,
+    PhpClassRequirement,
+    PHPExtensionRequirement,
+    PhpPackageRequirement,
+    PkgConfigRequirement,
+    PytestPluginRequirement,
     PythonModuleRequirement,
     PythonPackageRequirement,
-    CertificateAuthorityRequirement,
-    LibtoolRequirement,
+    QtModuleRequirement,
+    QTRequirement,
+    Requirement,
+    RPackageRequirement,
+    RubyFileRequirement,
+    RubyGemRequirement,
+    SprocketsFileRequirement,
+    StaticLibraryRequirement,
     VagueDependencyRequirement,
-    PerlPreDeclaredRequirement,
-    IntrospectionTypelibRequirement,
-    PHPExtensionRequirement,
+    ValaPackageRequirement,
     VcsControlDirectoryAccessRequirement,
+    X11Requirement,
+    XmlEntityRequirement,
 )
+from . import Resolver, UnsatisfiedRequirements
 
 
 class AptRequirement(Requirement):
-
     family = "apt"
 
-    def __init__(self, relations):
-        super(AptRequirement, self).__init__()
+    def __init__(self, relations) -> None:
+        super().__init__()
         if not isinstance(relations, list):
             raise TypeError(relations)
         self.relations = relations
@@ -116,15 +116,15 @@ class AptRequirement(Requirement):
         return hash((type(self), self.pkg_relation_str()))
 
     def __eq__(self, other):
-        return (isinstance(self, type(other))
-                and self.relations == other.relations)
+        return (
+            isinstance(self, type(other)) and self.relations == other.relations
+        )
 
-    def __str__(self):
-        return "apt requirement: %s" % self.pkg_relation_str()
+    def __str__(self) -> str:
+        return f"apt requirement: {self.pkg_relation_str()}"
 
-    def __repr__(self):
-        return "%s.from_str(%r)" % (
-            type(self).__name__, self.pkg_relation_str())
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}.from_str({self.pkg_relation_str()!r})"
 
     def package_names(self):
         for rel in self.relations:
@@ -137,19 +137,22 @@ class AptRequirement(Requirement):
     def satisfied_by(self, binaries, version):
         def binary_pkg_matches(entry, binary):
             # TODO(jelmer): check versions
-            if entry['name'] == binary['Package']:
+            if entry["name"] == binary["Package"]:
                 return True
-            for provides_top in PkgRelation.parse_relations(
-                    binary.get('Provides', '')):
+            provides_str = binary.get("Provides", "").replace(
+                "${binary:Version}", str(version)
+            )
+            for provides_top in PkgRelation.parse_relations(provides_str):
                 for provides in provides_top:
-                    if entry['name'] == provides['name']:
+                    if entry["name"] == provides["name"]:
                         return True
             return False
 
         for rel in self.relations:
             for entry in rel:
-                if any(binary_pkg_matches(entry, binary)
-                       for binary in binaries):
+                if any(
+                    binary_pkg_matches(entry, binary) for binary in binaries
+                ):
                     break
             else:
                 return False
@@ -161,15 +164,18 @@ async def resolve_perl_predeclared_req(apt_mgr, req):
         req = req.lookup_module()
     except KeyError:
         logging.warning(
-            'Unable to map predeclared function %s to a perl module', req.name)
+            "Unable to map predeclared function %s to a perl module", req.name
+        )
         return None
     return await resolve_perl_module_req(apt_mgr, req)
 
 
 async def find_package_names(
-    apt_mgr: AptManager, paths: List[str], regex: bool = False,
-    case_insensitive: bool = False
-) -> List[str]:
+    apt_mgr: AptManager,
+    paths: list[str],
+    regex: bool = False,
+    case_insensitive: bool = False,
+) -> list[str]:
     if not isinstance(paths, list):
         raise TypeError(paths)
     return await apt_mgr.get_packages_for_paths(paths, regex, case_insensitive)
@@ -177,17 +183,18 @@ async def find_package_names(
 
 async def find_reqs_simple(
     apt_mgr: AptManager,
-    paths: List[str],
+    paths: list[str],
     regex: bool = False,
     minimum_version=None,
     case_insensitive=False,
-) -> List[str]:
+) -> list[str]:
     if not isinstance(paths, list):
         raise TypeError(paths)
     return [
         AptRequirement.simple(package, minimum_version=minimum_version)
         for package in await find_package_names(
-            apt_mgr, paths, regex, case_insensitive)
+            apt_mgr, paths, regex, case_insensitive
+        )
     ]
 
 
@@ -208,25 +215,45 @@ def python_spec_to_apt_rels(pkg_name, specs):
                 next_maj_deb_version = Version(".".join(parts))
                 deb_version = Version(spec[1])
                 rels.extend(
-                    [[{"name": pkg_name,
-                       "version": (">=", deb_version)}],
-                     [{"name": pkg_name,
-                       "version": ("<<", next_maj_deb_version)}]])
+                    [
+                        [{"name": pkg_name, "version": (">=", deb_version)}],
+                        [
+                            {
+                                "name": pkg_name,
+                                "version": ("<<", next_maj_deb_version),
+                            }
+                        ],
+                    ]
+                )
             elif spec[0] == "!=":
                 deb_version = Version(spec[1])
-                rels.extend([
-                    [{"name": pkg_name, "version": (">>", deb_version)}],
-                    [{"name": pkg_name, "version": ("<<", deb_version)}]])
+                rels.extend(
+                    [
+                        [{"name": pkg_name, "version": (">>", deb_version)}],
+                        [{"name": pkg_name, "version": ("<<", deb_version)}],
+                    ]
+                )
             elif spec[1].endswith(".*") and spec[0] == "==":
                 s = spec[1].split(".")
                 s.pop(-1)
                 n = list(s)
                 n[-1] = str(int(n[-1]) + 1)
                 rels.extend(
-                    [[{"name": pkg_name,
-                       "version": (">=", Version(".".join(s)))}],
-                     [{"name": pkg_name,
-                       "version": ("<<", Version(".".join(n)))}]])
+                    [
+                        [
+                            {
+                                "name": pkg_name,
+                                "version": (">=", Version(".".join(s))),
+                            }
+                        ],
+                        [
+                            {
+                                "name": pkg_name,
+                                "version": ("<<", Version(".".join(n))),
+                            }
+                        ],
+                    ]
+                )
             else:
                 c = {
                     ">=": ">=",
@@ -244,15 +271,14 @@ async def get_package_for_python_package(
     apt_mgr, package, python_version: Optional[str], specs=None
 ):
     pypy_regex = (
-        "/usr/lib/pypy/dist\\-packages/%s-.*\\.(dist|egg)\\-info"
-        % re.escape(package.replace("-", "_")))
+        "/usr/lib/pypy/dist\\-packages/{}-.*\\.(dist|egg)\\-info".format(re.escape(package.replace("-", "_")))
+    )
     cpython2_regex = (
-        "/usr/lib/python2\\.[0-9]/dist\\-packages/%s-.*\\.(dist|egg)\\-info"
-        % re.escape(package.replace("-", "_"))
+        "/usr/lib/python2\\.[0-9]/dist\\-packages/{}-.*\\.(dist|egg)\\-info".format(re.escape(package.replace("-", "_")))
     )
     cpython3_regex = (
-        "/usr/lib/python3/dist\\-packages/%s-.*\\.(dist|egg)\\-info"
-        % re.escape(package.replace("-", "_")))
+        "/usr/lib/python3/dist\\-packages/{}-.*\\.(dist|egg)\\-info".format(re.escape(package.replace("-", "_")))
+    )
     if python_version == "pypy":
         paths = [pypy_regex]
     elif python_version == "cpython2":
@@ -263,43 +289,49 @@ async def get_package_for_python_package(
         paths = [cpython3_regex, cpython2_regex, pypy_regex]
     else:
         raise NotImplementedError(
-            "unsupported python version %s" % python_version)
+            f"unsupported python version {python_version}"
+        )
     names = await find_package_names(
-        apt_mgr, paths, regex=True, case_insensitive=True)
-    return [AptRequirement(
-        python_spec_to_apt_rels(name, specs)) for name in names]
+        apt_mgr, paths, regex=True, case_insensitive=True
+    )
+    return [
+        AptRequirement(python_spec_to_apt_rels(name, specs)) for name in names
+    ]
 
 
 def get_possible_python3_paths_for_python_object(object_path):
     cpython3_regexes = []
     while True:
-        cpython3_regexes.extend([
-            posixpath.join(
-                re.escape("/usr/lib/python3/dist-packages"),
-                re.escape(object_path.replace(".", "/")),
-                re.escape("__init__.py"),
-            ),
-            posixpath.join(
-                re.escape("/usr/lib/python3/dist-packages"),
-                re.escape(object_path.replace(".", "/")) + re.escape(".py"),
-            ),
-            posixpath.join(
-                "/usr/lib/python3\\.[0-9]+/lib\\-dynload",
-                re.escape(object_path.replace(".", "/"))
-                + ".cpython\\-.*\\.so",
-            ),
-            posixpath.join(
-                "/usr/lib/python3\\.[0-9]+/",
-                re.escape(object_path.replace(".", "/")) + "\\.py"
-            ),
-            posixpath.join(
-                "/usr/lib/python3\\.[0-9]+/",
-                re.escape(object_path.replace(".", "/")),
-                "__init__\\.py",
-            ),
-        ])
+        cpython3_regexes.extend(
+            [
+                posixpath.join(
+                    re.escape("/usr/lib/python3/dist-packages"),
+                    re.escape(object_path.replace(".", "/")),
+                    re.escape("__init__.py"),
+                ),
+                posixpath.join(
+                    re.escape("/usr/lib/python3/dist-packages"),
+                    re.escape(object_path.replace(".", "/"))
+                    + re.escape(".py"),
+                ),
+                posixpath.join(
+                    "/usr/lib/python3\\.[0-9]+/lib\\-dynload",
+                    re.escape(object_path.replace(".", "/"))
+                    + ".cpython\\-.*\\.so",
+                ),
+                posixpath.join(
+                    "/usr/lib/python3\\.[0-9]+/",
+                    re.escape(object_path.replace(".", "/")) + "\\.py",
+                ),
+                posixpath.join(
+                    "/usr/lib/python3\\.[0-9]+/",
+                    re.escape(object_path.replace(".", "/")),
+                    "__init__\\.py",
+                ),
+            ]
+        )
         try:
-            object_path, discarded = object_path.rsplit('.', 1)
+            object_path, discarded = object_path.rsplit(".", 1)
         except ValueError:
             break
     return cpython3_regexes
@@ -308,23 +340,26 @@ def get_possible_python3_paths_for_python_object(object_path):
 def get_possible_pypy_paths_for_python_object(object_path):
     pypy_regexes = []
     while True:
-        pypy_regexes.extend([
-            posixpath.join(
-                "/usr/lib/pypy/dist\\-packages",
-                re.escape(object_path.replace(".", "/")),
-                "__init__\\.py",
-            ),
-            posixpath.join(
-                "/usr/lib/pypy/dist\\-packages",
-                re.escape(object_path.replace(".", "/")) + "\\.py"
-            ),
-            posixpath.join(
-                "/usr/lib/pypy/dist\\-packages",
-                re.escape(object_path.replace(".", "/")) + "\\.pypy-.*\\.so",
-            ),
-        ])
+        pypy_regexes.extend(
+            [
+                posixpath.join(
+                    "/usr/lib/pypy/dist\\-packages",
+                    re.escape(object_path.replace(".", "/")),
+                    "__init__\\.py",
+                ),
+                posixpath.join(
+                    "/usr/lib/pypy/dist\\-packages",
+                    re.escape(object_path.replace(".", "/")) + "\\.py",
+                ),
+                posixpath.join(
+                    "/usr/lib/pypy/dist\\-packages",
+                    re.escape(object_path.replace(".", "/"))
+                    + "\\.pypy-.*\\.so",
+                ),
+            ]
+        )
         try:
-            object_path, discarded = object_path.rsplit('.', 1)
+            object_path, discarded = object_path.rsplit(".", 1)
         except ValueError:
             break
     return pypy_regexes
@@ -333,30 +368,33 @@ def get_possible_pypy_paths_for_python_object(object_path):
 def get_possible_python2_paths_for_python_object(object_path):
     cpython2_regexes = []
     while True:
-        cpython2_regexes.extend([
-            posixpath.join(
-                "/usr/lib/python2\\.[0-9]/dist\\-packages",
-                re.escape(object_path.replace(".", "/")),
-                "__init__\\.py",
-            ),
-            posixpath.join(
-                "/usr/lib/python2\\.[0-9]/dist\\-packages",
-                re.escape(object_path.replace(".", "/")) + "\\.py",
-            ),
-            posixpath.join(
-                "/usr/lib/python2.\\.[0-9]/lib\\-dynload",
-                re.escape(object_path.replace(".", "/")) + "\\.so",
-            ),
-        ])
+        cpython2_regexes.extend(
+            [
+                posixpath.join(
+                    "/usr/lib/python2\\.[0-9]/dist\\-packages",
+                    re.escape(object_path.replace(".", "/")),
+                    "__init__\\.py",
+                ),
+                posixpath.join(
+                    "/usr/lib/python2\\.[0-9]/dist\\-packages",
+                    re.escape(object_path.replace(".", "/")) + "\\.py",
+                ),
+                posixpath.join(
+                    "/usr/lib/python2.\\.[0-9]/lib\\-dynload",
+                    re.escape(object_path.replace(".", "/")) + "\\.so",
+                ),
+            ]
+        )
         try:
-            object_path, discarded = object_path.rsplit('.', 1)
+            object_path, discarded = object_path.rsplit(".", 1)
         except ValueError:
             break
     return cpython2_regexes
 
 
 async def get_package_for_python_object_path(
-        apt_mgr, object_path, python_version, specs):
+    apt_mgr, object_path, python_version, specs
+):
     # Try to find the most specific file
     if python_version == "cpython3":
         paths = get_possible_python3_paths_for_python_object(object_path)
@@ -365,14 +403,17 @@ async def get_package_for_python_object_path(
     elif python_version == "pypy":
         paths = get_possible_pypy_paths_for_python_object(object_path)
     elif python_version is None:
-        paths = (get_possible_python3_paths_for_python_object(object_path)
-                 + get_possible_python2_paths_for_python_object(object_path)
-                 + get_possible_pypy_paths_for_python_object(object_path))
+        paths = (
+            get_possible_python3_paths_for_python_object(object_path)
+            + get_possible_python2_paths_for_python_object(object_path)
+            + get_possible_pypy_paths_for_python_object(object_path)
+        )
     else:
-        raise AssertionError("unknown python version %r" % python_version)
+        raise AssertionError(f"unknown python version {python_version!r}")
     names = await find_package_names(apt_mgr, paths, regex=True)
-    return [AptRequirement(python_spec_to_apt_rels(name, specs))
-            for name in names]
+    return [
+        AptRequirement(python_spec_to_apt_rels(name, specs)) for name in names
+    ]
 
 
 vague_map = {
@@ -380,7 +421,6 @@ vague_map = {
     "the required FreeType library": "libfreetype-dev",
     "the Boost C++ libraries": "libboost-dev",
     "the sndfile library": "libsndfile-dev",
-
     # TODO(jelmer): Support resolving virtual packages
     "PythonLibs": "libpython3-dev",
     "PythonInterp": "python3",
@@ -388,7 +428,6 @@ vague_map = {
     "Osmium": "libosmium2-dev",
     "glib": "libglib2.0-dev",
     "OpenGL": "libgl-dev",
-
     # TODO(jelmer): For Python, check minimum_version and map to python 2 or
     # python 3
     "Python": "libpython3-dev",
@@ -399,53 +438,75 @@ vague_map = {
 async def resolve_vague_dep_req(apt_mgr, req):
     name = req.name
     options = []
-    if ' or ' in name:
-        for entry in name.split(' or '):
-            options.extend(await resolve_vague_dep_req(
-                apt_mgr, VagueDependencyRequirement(entry)))
+    if " or " in name:
+        for entry in name.split(" or "):
+            options.extend(
+                await resolve_vague_dep_req(
+                    apt_mgr, VagueDependencyRequirement(entry)
+                )
+            )
 
     if name in vague_map:
-        options.append(AptRequirement.simple(
-            vague_map[name], minimum_version=req.minimum_version))
+        options.append(
+            AptRequirement.simple(
+                vague_map[name], minimum_version=req.minimum_version
+            )
+        )
     for x in req.expand():
         options.extend(await resolve_requirement_apt(apt_mgr, x))
 
-    if name.startswith('GNU '):
-        options.extend(await resolve_vague_dep_req(
-            apt_mgr, VagueDependencyRequirement(name[4:])))
+    if name.startswith("GNU "):
+        options.extend(
+            await resolve_vague_dep_req(
+                apt_mgr, VagueDependencyRequirement(name[4:])
+            )
+        )
 
-    if name.startswith('py') or name.endswith('py'):
+    if name.startswith("py") or name.endswith("py"):
         # TODO(jelmer): Try harder to determine whether this is a python
         # package
-        options.append(await resolve_requirement_apt(
-            apt_mgr, PythonPackageRequirement(name)))
+        options.append(
+            await resolve_requirement_apt(
+                apt_mgr, PythonPackageRequirement(name)
+            )
+        )
 
     # Try even harder
     if not options:
-        options.extend(await find_reqs_simple(
-            apt_mgr,
-            [
-                posixpath.join(
-                    "/usr/lib", ".*", "pkgconfig",
-                    re.escape(req.name) + "-.*\\.pc"),
-                posixpath.join(
-                    "/usr/lib/pkgconfig", re.escape(req.name) + "\\-.*\\.pc")
-            ],
-            regex=True,
-            case_insensitive=True,
-            minimum_version=req.minimum_version
-        ))
+        options.extend(
+            await find_reqs_simple(
+                apt_mgr,
+                [
+                    posixpath.join(
+                        "/usr/lib",
+                        ".*",
+                        "pkgconfig",
+                        re.escape(req.name) + "-.*\\.pc",
+                    ),
+                    posixpath.join(
+                        "/usr/lib/pkgconfig",
+                        re.escape(req.name) + "\\-.*\\.pc",
+                    ),
+                ],
+                regex=True,
+                case_insensitive=True,
+                minimum_version=req.minimum_version,
+            )
+        )
 
     return options
 
 
 async def resolve_php_extension_req(apt_mgr, req):
-    return [AptRequirement.simple("php-%s" % req.extension)]
+    return [AptRequirement.simple(f"php-{req.extension}")]
 
 
 async def resolve_octave_pkg_req(apt_mgr, req):
-    return [AptRequirement.simple(
-            "octave-%s" % req.package, minimum_version=req.minimum_version)]
+    return [
+        AptRequirement.simple(
+            f"octave-{req.package}", minimum_version=req.minimum_version
+        )
+    ]
 
 
 async def resolve_binary_req(apt_mgr, req):
@@ -502,7 +563,7 @@ async def resolve_js_runtime_req(apt_mgr, req):
 
 
 async def resolve_vala_package_req(apt_mgr, req):
-    path = "/usr/share/vala-[0-9.]+/vapi/%s\\.vapi" % re.escape(req.package)
+    path = f"/usr/share/vala-[0-9.]+/vapi/{re.escape(req.package)}\\.vapi"
     return await find_reqs_simple(apt_mgr, [path], regex=True)
 
 
@@ -510,7 +571,7 @@ async def resolve_ruby_gem_req(apt_mgr, req):
     paths = [
         posixpath.join(
             "/usr/share/rubygems-integration/all/"
-            "specifications/%s-.*\\.gemspec" % re.escape(req.gem)
+            f"specifications/{re.escape(req.gem)}-.*\\.gemspec"
         )
     ]
     return await find_reqs_simple(
@@ -521,8 +582,11 @@ async def resolve_ruby_gem_req(apt_mgr, req):
 async def resolve_go_package_req(apt_mgr, req):
     return await find_reqs_simple(
         apt_mgr,
-        [posixpath.join(
-            "/usr/share/gocode/src", re.escape(req.package), ".*")],
+        [
+            posixpath.join(
+                "/usr/share/gocode/src", re.escape(req.package), ".*"
+            )
+        ],
         regex=True,
     )
 
@@ -530,7 +594,9 @@ async def resolve_go_package_req(apt_mgr, req):
 async def resolve_go_req(apt_mgr, req):
     return [
         AptRequirement.simple(
-            "golang-go", minimum_version="2:%s~" % req.version)]
+            "golang-go", minimum_version=f"2:{req.version}~"
+        )
+    ]
 
 
 async def resolve_dh_addon_req(apt_mgr, req):
@@ -539,14 +605,15 @@ async def resolve_dh_addon_req(apt_mgr, req):
 
 
 async def resolve_php_class_req(apt_mgr, req):
-    path = "/usr/share/php/%s.php" % req.php_class.replace("\\", "/")
+    path = "/usr/share/php/{}.php".format(req.php_class.replace("\\", "/"))
     return await find_reqs_simple(apt_mgr, [path])
 
 
 async def resolve_php_package_req(apt_mgr, req):
     return [
         AptRequirement.simple(
-            "php-%s" % req.package, minimum_version=req.min_version)
+            f"php-{req.package}", minimum_version=req.min_version
+        )
     ]
 
 
@@ -555,56 +622,55 @@ async def resolve_r_package_req(apt_mgr, req):
         posixpath.join("/usr/lib/R/site-library", req.package, "DESCRIPTION")
     ]
     return await find_reqs_simple(
-        apt_mgr, paths, minimum_version=req.minimum_version)
+        apt_mgr, paths, minimum_version=req.minimum_version
+    )
 
 
 async def resolve_node_module_req(apt_mgr, req):
     paths = [
-        "/usr/share/nodejs/.*/node_modules/%s/index\\.js"
-        % re.escape(req.module),
-        "/usr/lib/nodejs/%s/index\\.js" % re.escape(req.module),
-        "/usr/share/nodejs/%s/index\\.js" % re.escape(req.module),
+        f"/usr/share/nodejs/.*/node_modules/{re.escape(req.module)}/index\\.js",
+        f"/usr/lib/nodejs/{re.escape(req.module)}/index\\.js",
+        f"/usr/share/nodejs/{re.escape(req.module)}/index\\.js",
     ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
 
 
 async def resolve_node_package_req(apt_mgr, req):
     paths = [
-        "/usr/share/nodejs/.*/node_modules/%s/package\\.json"
-        % re.escape(req.package),
-        "/usr/lib/nodejs/%s/package\\.json" % re.escape(req.package),
-        "/usr/share/nodejs/%s/package\\.json" % re.escape(req.package),
+        f"/usr/share/nodejs/.*/node_modules/{re.escape(req.package)}/package\\.json",
+        f"/usr/lib/nodejs/{re.escape(req.package)}/package\\.json",
+        f"/usr/share/nodejs/{re.escape(req.package)}/package\\.json",
     ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
 
 
 async def resolve_library_req(apt_mgr, req):
     paths = [
-        posixpath.join("/usr/lib/lib%s\\.so$" % re.escape(req.library)),
-        posixpath.join("/usr/lib/.*/lib%s\\.so$" % re.escape(req.library)),
-        posixpath.join("/usr/lib/lib%s\\.a$" % re.escape(req.library)),
-        posixpath.join("/usr/lib/.*/lib%s\\.a$" % re.escape(req.library)),
+        posixpath.join(f"/usr/lib/lib{re.escape(req.library)}\\.so$"),
+        posixpath.join(f"/usr/lib/.*/lib{re.escape(req.library)}\\.so$"),
+        posixpath.join(f"/usr/lib/lib{re.escape(req.library)}\\.a$"),
+        posixpath.join(f"/usr/lib/.*/lib{re.escape(req.library)}\\.a$"),
     ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
 
 
 async def resolve_static_library_req(apt_mgr, req):
     paths = [
-        posixpath.join("/usr/lib/%s$" % re.escape(req.filename)),
-        posixpath.join("/usr/lib/.*/%s$" % re.escape(req.filename)),
+        posixpath.join(f"/usr/lib/{re.escape(req.filename)}$"),
+        posixpath.join(f"/usr/lib/.*/{re.escape(req.filename)}$"),
     ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
 
 
 async def resolve_ruby_file_req(apt_mgr, req):
-    paths = [posixpath.join("/usr/lib/ruby/vendor_ruby/%s.rb" % req.filename)]
+    paths = [posixpath.join(f"/usr/lib/ruby/vendor_ruby/{req.filename}.rb")]
     reqs = await find_reqs_simple(apt_mgr, paths, regex=False)
     if reqs:
         return reqs
     paths = [
         posixpath.join(
             r"/usr/share/rubygems\-integration/all/gems/([^/]+)/"
-            "lib/%s\\.rb" % re.escape(req.filename)
+            f"lib/{re.escape(req.filename)}\\.rb"
         )
     ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
@@ -614,12 +680,11 @@ async def resolve_xml_entity_req(apt_mgr, req):
     # Ideally we should be using the XML catalog for this, but hardcoding
     # a few URLs will do for now..
     URL_MAP = {
-        "http://www.oasis-open.org/docbook/xml/":
-            "/usr/share/xml/docbook/schema/dtd/"
+        "http://www.oasis-open.org/docbook/xml/": "/usr/share/xml/docbook/schema/dtd/"
     }
     for url, path in URL_MAP.items():
         if req.url.startswith(url):
-            search_path = posixpath.join(path, req.url[len(url):])
+            search_path = posixpath.join(path, req.url[len(url) :])
             break
     else:
         return None
@@ -629,11 +694,11 @@ async def resolve_xml_entity_req(apt_mgr, req):
 
 async def resolve_sprockets_file_req(apt_mgr, req):
     if req.content_type == "application/javascript":
-        path = ("/usr/share/.*/app/assets/javascripts/%s\\.js$"
-                % re.escape(req.name))
+        path = "/usr/share/.*/app/assets/javascripts/{}\\.js$".format(re.escape(
+            req.name
+        ))
     else:
-        logging.warning(
-            "unable to handle content type %s", req.content_type)
+        logging.warning("unable to handle content type %s", req.content_type)
         return None
     return await find_reqs_simple(apt_mgr, [path], regex=True)
 
@@ -652,14 +717,17 @@ async def resolve_java_class_req(apt_mgr, req):
 
 
 async def resolve_cmake_file_req(apt_mgr, req):
-    paths = ['/usr/lib/.*/cmake/.*/%s' % re.escape(req.filename),
-             '/usr/share/.*/cmake/%s' % re.escape(req.filename)]
+    paths = [
+        f"/usr/lib/.*/cmake/.*/{re.escape(req.filename)}",
+        f"/usr/share/.*/cmake/{re.escape(req.filename)}",
+    ]
     return await find_reqs_simple(apt_mgr, paths, regex=True)
 
 
 async def resolve_haskell_package_req(apt_mgr, req):
-    path = ("/var/lib/ghc/package\\.conf\\.d/%s\\-.*\\.conf"
-            % re.escape(req.package))
+    path = "/var/lib/ghc/package\\.conf\\.d/{}\\-.*\\.conf".format(re.escape(
+        req.package
+    ))
     return await find_reqs_simple(apt_mgr, [path], regex=True)
 
 
@@ -681,7 +749,7 @@ async def resolve_maven_artifact_req(apt_mgr, req):
         escape(req.group_id.replace(".", "/")),
         escape(req.artifact_id),
         version,
-        escape("%s-" % req.artifact_id) + version + escape("." + kind),
+        escape(f"{req.artifact_id}-") + version + escape("." + kind),
     )
 
     return await find_reqs_simple(apt_mgr, [path], regex=regex)
@@ -710,15 +778,18 @@ async def resolve_x11_req(apt_mgr, req):
 
 async def resolve_qt_req(apt_mgr, req):
     return await find_reqs_simple(
-        apt_mgr, ["/usr/lib/.*/qt[0-9]+/bin/qmake"], regex=True)
+        apt_mgr, ["/usr/lib/.*/qt[0-9]+/bin/qmake"], regex=True
+    )
 
 
 async def resolve_qt_module_req(apt_mgr, req):
     return await find_reqs_simple(
         apt_mgr,
-        ["/usr/lib/.*/qt5/mkspecs/modules/qt_lib_%s\\.pri"
-            % re.escape(req.module)],
-        regex=True)
+        [
+            f"/usr/lib/.*/qt5/mkspecs/modules/qt_lib_{re.escape(req.module)}\\.pri"
+        ],
+        regex=True,
+    )
 
 
 async def resolve_libtool_req(apt_mgr, req):
@@ -727,20 +798,27 @@ async def resolve_libtool_req(apt_mgr, req):
 
 async def resolve_perl_module_req(apt_mgr, req):
     DEFAULT_PERL_PATHS = [
-        "/usr/share/perl5", "/usr/lib/.*/perl5/.*", "/usr/lib/.*/perl-base",
-        "/usr/lib/.*/perl/[^/]+", "/usr/share/perl/[^/]+"]
+        "/usr/share/perl5",
+        "/usr/lib/.*/perl5/.*",
+        "/usr/lib/.*/perl-base",
+        "/usr/lib/.*/perl/[^/]+",
+        "/usr/share/perl/[^/]+",
+    ]
 
     if req.inc is None:
         if req.filename is None:
             paths = [
                 posixpath.join(
-                    inc, re.escape(req.module.replace('::', '/') + '.pm'))
-                for inc in DEFAULT_PERL_PATHS]
+                    inc, re.escape(req.module.replace("::", "/") + ".pm")
+                )
+                for inc in DEFAULT_PERL_PATHS
+            ]
             regex = True
         elif not posixpath.isabs(req.filename):
             paths = [
                 posixpath.join(inc, re.escape(req.filename))
-                for inc in DEFAULT_PERL_PATHS]
+                for inc in DEFAULT_PERL_PATHS
+            ]
             regex = True
         else:
             paths = [req.filename]
@@ -756,16 +834,16 @@ async def resolve_perl_file_req(apt_mgr, req):
 
 
 def _m4_macro_regex(macro):
-    defun_prefix = re.escape("AC_DEFUN([%s]," % macro)
-    au_alias_prefix = re.escape("AU_ALIAS([%s]," % macro)
-    m4_copy = r"m4_copy\(.*,\s*\[%s\]\)" % re.escape(macro)
+    defun_prefix = re.escape(f"AC_DEFUN([{macro}],")
+    au_alias_prefix = re.escape(f"AU_ALIAS([{macro}],")
+    m4_copy = rf"m4_copy\(.*,\s*\[{re.escape(macro)}\]\)"
     return "(" + "|".join([defun_prefix, au_alias_prefix, m4_copy]) + ")"
 
 
 def _find_local_m4_macro(macro):
     # TODO(jelmer): Query some external service that can search all binary
     # packages?
-    p = re.compile(_m4_macro_regex(macro).encode('ascii'))
+    p = re.compile(_m4_macro_regex(macro).encode("ascii"))
     for entry in os.scandir("/usr/share/aclocal"):
         if not entry.is_file():
             continue
@@ -792,10 +870,12 @@ async def resolve_python_module_req(apt_mgr, req):
         specs = []
     if req.python_version == 2:
         return await get_package_for_python_object_path(
-            apt_mgr, req.module, "cpython2", specs)
+            apt_mgr, req.module, "cpython2", specs
+        )
     elif req.python_version in (None, 3):
         return await get_package_for_python_object_path(
-            apt_mgr, req.module, "cpython3", specs)
+            apt_mgr, req.module, "cpython3", specs
+        )
     else:
         return None
 
@@ -815,10 +895,11 @@ async def resolve_python_package_req(apt_mgr, req):
 
 async def resolve_cargo_crate_req(apt_mgr, req):
     paths = [
-        "/usr/share/cargo/registry/%s\\-[0-9]+.*/Cargo\\.toml"
-        % re.escape(req.crate)]
+        f"/usr/share/cargo/registry/{re.escape(req.crate)}\\-[0-9]+.*/Cargo\\.toml"
+    ]
     return await find_reqs_simple(
-        apt_mgr, paths, regex=True, minimum_version=req.minimum_version)
+        apt_mgr, paths, regex=True, minimum_version=req.minimum_version
+    )
 
 
 async def resolve_ca_req(apt_mgr, req):
@@ -828,9 +909,11 @@ async def resolve_ca_req(apt_mgr, req):
 async def resolve_introspection_typelib_req(apt_mgr, req):
     return await find_reqs_simple(
         apt_mgr,
-        [r'/usr/lib/.*/girepository\\-.*/%s\\-.*\.typelib'
-            % re.escape(req.library)],
-        regex=True)
+        [
+            rf"/usr/lib/.*/girepository\\-.*/{re.escape(req.library)}\\-.*\.typelib"
+        ],
+        regex=True,
+    )
 
 
 async def resolve_apt_req(apt_mgr, req):
@@ -843,45 +926,56 @@ async def resolve_apt_req(apt_mgr, req):
 
 async def resolve_boost_component_req(apt_mgr, req):
     return await find_reqs_simple(
-        apt_mgr, ["/usr/lib/.*/libboost_%s" % re.escape(req.name)],
-        regex=True)
+        apt_mgr, [f"/usr/lib/.*/libboost_{re.escape(req.name)}"], regex=True
+    )
 
 
 async def resolve_kf5_component_req(apt_mgr, req):
     return await find_reqs_simple(
-        apt_mgr, ["/usr/lib/.*/cmake/KF5%s/KF5%sConfig\\.cmake" % (
-            re.escape(req.name), re.escape(req.name))],
-        regex=True)
+        apt_mgr,
+        [
+            f"/usr/lib/.*/cmake/KF5{re.escape(req.name)}/KF5{re.escape(req.name)}Config\\.cmake"
+        ],
+        regex=True,
+    )
+
+
+async def resolve_pytest_plugin_req(apt_mgr, req):
+    return [AptRequirement.simple(f"python3-pytest-{req.plugin}")]
 
 
 async def resolve_vcs_access_req(apt_mgr, req):
     PKG_MAP = {
-        'hg': 'mercurial',
-        'svn': 'subversion',
-        'git': 'git',
-        'bzr': 'bzr',
-        }
+        "hg": "mercurial",
+        "svn": "subversion",
+        "git": "git",
+        "bzr": "bzr",
+    }
     ret = []
     for vcs in req.vcs:
         try:
             ret.append(PKG_MAP[vcs])
         except KeyError:
-            logging.debug('Unable to map VCS %s to package', vcs)
-    return [AptRequirement.from_str(','.join(ret))]
+            logging.debug("Unable to map VCS %s to package", vcs)
+    return [AptRequirement.from_str(",".join(ret))]
 
 
 async def resolve_oneof_req(apt_mgr, req):
     options = await asyncio.gather(
-        *[resolve_requirement_apt(apt_mgr, req) for req in req.elements])
+        *[resolve_requirement_apt(apt_mgr, req) for req in req.elements]
+    )
     for option in options:
         if not option:
             continue
         return option
 
 
-APT_REQUIREMENT_RESOLVERS: List[Tuple[
-        Type[Requirement], Callable[
-            [AptManager, Requirement], Awaitable[List[AptRequirement]]]]] = [
+APT_REQUIREMENT_RESOLVERS: list[
+    tuple[
+        type[Requirement],
+        Callable[[AptManager, Requirement], Awaitable[list[AptRequirement]]],
+    ]
+] = [
     (AptRequirement, resolve_apt_req),
     (BinaryRequirement, resolve_binary_req),
     (VagueDependencyRequirement, resolve_vague_dep_req),
@@ -930,12 +1024,14 @@ APT_REQUIREMENT_RESOLVERS: List[Tuple[
     (PHPExtensionRequirement, resolve_php_extension_req),
     (OctavePackageRequirement, resolve_octave_pkg_req),
     (VcsControlDirectoryAccessRequirement, resolve_vcs_access_req),
+    (PytestPluginRequirement, resolve_pytest_plugin_req),
     (OneOfRequirement, resolve_oneof_req),
 ]
 
 
 async def resolve_requirement_apt(
-        apt_mgr, req: Requirement) -> List[AptRequirement]:
+    apt_mgr, req: Requirement
+) -> list[AptRequirement]:
     for rr_class, rr_fn in APT_REQUIREMENT_RESOLVERS:
         if isinstance(req, rr_class):
             ret = await rr_fn(apt_mgr, req)
@@ -943,35 +1039,38 @@ async def resolve_requirement_apt(
                 return []
             if not isinstance(ret, list):
                 raise TypeError(ret)
+            logging.debug(
+                "Trying to resolve %r by installing one of %r", req, ret
+            )
             return ret
     logging.warning(
-        'No way known to convert %s to apt requirement',
-        type(req).__name__)
+        "No way known to convert %s to apt requirement", type(req).__name__
+    )
     return []
 
 
 def default_tie_breakers(session):
-    from ..debian.udd import popcon_tie_breaker
     from ..debian.build_deps import BuildDependencyTieBreaker
+    from ..debian.udd import popcon_tie_breaker
+
     return [
         BuildDependencyTieBreaker.from_session(session),
         popcon_tie_breaker,
-        ]
+    ]
 
 
 class AptResolver(Resolver):
-    def __init__(self, apt, tie_breakers=None):
+    def __init__(self, apt, tie_breakers=None) -> None:
         self.apt = apt
         if tie_breakers is None:
             tie_breakers = default_tie_breakers(apt.session)
         self.tie_breakers = tie_breakers
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "apt"
 
-    def __repr__(self):
-        return "%s(%r, %r)" % (
-            type(self).__name__, self.apt, self.tie_breakers)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.apt!r}, {self.tie_breakers!r})"
 
     @classmethod
     def from_session(cls, session, tie_breakers=None):
@@ -1009,7 +1108,8 @@ class AptResolver(Resolver):
         if apt_requirements:
             yield (
                 self.apt.satisfy_command(
-                    [r.pkg_relation_str() for o, r in apt_requirements]),
+                    [r.pkg_relation_str() for o, r in apt_requirements]
+                ),
                 [o for o, r in apt_requirements],
             )
 
@@ -1023,7 +1123,8 @@ class AptResolver(Resolver):
         if len(ret) == 1:
             return ret[0]
         logging.info(
-            "Need to break tie between %r with %r", ret, self.tie_breakers)
+            "Need to break tie between %r with %r", ret, self.tie_breakers
+        )
         for tie_breaker in self.tie_breakers:
             winner = tie_breaker(ret)
             if winner is not None:
@@ -1031,5 +1132,6 @@ class AptResolver(Resolver):
                     raise TypeError(winner)
                 return winner
         logging.info(
-            "Unable to break tie over %r, picking first: %r", ret, ret[0])
+            "Unable to break tie over %r, picking first: %r", ret, ret[0]
+        )
         return ret[0]
