@@ -16,22 +16,22 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from typing import Optional, List, Dict, Tuple
-import sys
 import subprocess
+import sys
+from typing import Optional
 
 
 class NoSessionOpen(Exception):
     """There is no session open."""
 
-    def __init__(self, session):
+    def __init__(self, session) -> None:
         self.session = session
 
 
 class SessionAlreadyOpen(Exception):
     """There is already a session open."""
 
-    def __init__(self, session):
+    def __init__(self, session) -> None:
         self.session = session
 
 
@@ -51,32 +51,37 @@ class Session:
 
     def check_call(
         self,
-        argv: List[str],
+        argv: list[str],
         cwd: Optional[str] = None,
         user: Optional[str] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
         close_fds: bool = True,
     ):
         raise NotImplementedError(self.check_call)
 
     def check_output(
         self,
-        argv: List[str],
+        argv: list[str],
         cwd: Optional[str] = None,
         user: Optional[str] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
     ) -> bytes:
         raise NotImplementedError(self.check_output)
 
     def Popen(
-        self, argv, cwd: Optional[str] = None, user: Optional[str] = None,
-        **kwargs
+        self,
+        argv,
+        cwd: Optional[str] = None,
+        user: Optional[str] = None,
+        **kwargs,
     ):
         raise NotImplementedError(self.Popen)
 
     def call(
-        self, argv: List[str], cwd: Optional[str] = None,
-        user: Optional[str] = None
+        self,
+        argv: list[str],
+        cwd: Optional[str] = None,
+        user: Optional[str] = None,
     ):
         raise NotImplementedError(self.call)
 
@@ -93,10 +98,10 @@ class Session:
 
     def setup_from_vcs(
         self, tree, include_controldir: Optional[bool] = None, subdir="package"
-    ) -> Tuple[str, str]:
+    ) -> tuple[str, str]:
         raise NotImplementedError(self.setup_from_vcs)
 
-    def setup_from_directory(self, path, subdir="package") -> Tuple[str, str]:
+    def setup_from_directory(self, path, subdir="package") -> tuple[str, str]:
         raise NotImplementedError(self.setup_from_directory)
 
     def external_path(self, path: str) -> str:
@@ -111,17 +116,19 @@ class Session:
 class SessionSetupFailure(Exception):
     """Session failed to be set up."""
 
-    def __init__(self, reason, errlines=None):
+    def __init__(self, reason, errlines=None) -> None:
         self.reason = reason
         self.errlines = errlines
 
 
-def run_with_tee(session: Session,
-                 args: List[str], **kwargs) -> Tuple[int, List[str]]:
+def run_with_tee(
+    session: Session, args: list[str], **kwargs
+) -> tuple[int, list[str]]:
     if "stdin" not in kwargs:
         kwargs["stdin"] = subprocess.DEVNULL
     p = session.Popen(
-        args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, **kwargs)
+        args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, **kwargs
+    )
     contents = []
     while p.poll() is None:
         line = p.stdout.readline()
@@ -132,8 +139,11 @@ def run_with_tee(session: Session,
 
 
 def get_user(session):
-    return session.check_output(
-        ["sh", "-c", "echo $USER"], cwd="/").decode().strip()
+    return (
+        session.check_output(["sh", "-c", "echo $USER"], cwd="/")
+        .decode()
+        .strip()
+    )
 
 
 def which(session, name):
