@@ -20,26 +20,37 @@ import os
 import shlex
 import subprocess
 import tempfile
+from typing import Optional
 
-from typing import Optional, List, Dict
+from .._ognibuild_rs import (
+    generate_session_id,
+    sanitize_session_name,
+)
+from . import NoSessionOpen, Session, SessionAlreadyOpen, SessionSetupFailure
 
-
-from . import Session, SessionSetupFailure, NoSessionOpen, SessionAlreadyOpen
+__all__ = [
+    "sanitize_session_name",
+    "generate_session_id",
+    "SchrootSession",
+]
 
 
 class SchrootSession(Session):
-
     _cwd: Optional[str]
     _location: Optional[str]
     chroot: str
     session_id: Optional[str]
+    session_prefix: Optional[str]
 
-    def __init__(self, chroot: str):
+    def __init__(
+        self, chroot: str, *, session_prefix: Optional[str] = None
+    ) -> None:
         if not isinstance(chroot, str):
-            raise TypeError("not a valid chroot: %r" % chroot)
+            raise TypeError(f"not a valid chroot: {chroot!r}")
         self.chroot = chroot
         self._location = None
         self._cwd = None
+        self._session_prefix = session_prefix
         self.session_id = None
 
     def _get_location(self) -> str:
@@ -67,7 +78,7 @@ class SchrootSession(Session):
                     logging.error("%s", line[3:].decode(errors="replace"))
             logging.warning(
                 "Failed to close schroot session %s, leaving stray.",
-                self.session_id
+                self.session_id,
             )
             self.session_id = None
             return False
@@ -79,10 +90,16 @@ class SchrootSession(Session):
         if self.session_id is not None:
             raise SessionAlreadyOpen(self)
         stderr = tempfile.TemporaryFile()
+        extra_args = []
+        if self._session_prefix:
+            sanitized_session_name = generate_session_id(self._session_prefix)
+            extra_args.extend(["-n", sanitized_session_name])
         try:
             self.session_id = (
                 subprocess.check_output(
-                    ["schroot", "-c", self.chroot, "-b"], stderr=stderr)
+                    ["schroot", "-c", self.chroot, "-b"] + extra_args,
+                    stderr=stderr,
+                )
                 .strip()
                 .decode()
             )
@@ -91,13 +108,16 @@ class SchrootSession(Session):
             errlines = stderr.readlines()
             if len(errlines) == 1:
                 raise SessionSetupFailure(
-                    errlines[0].rstrip().decode(), errlines=errlines) from e
+                    errlines[0].rstrip().decode(), errlines=errlines
+                ) from e
             elif len(errlines) == 0:
                 raise SessionSetupFailure(
-                    "No output from schroot", errlines=errlines) from e
+                    "No output from schroot", errlines=errlines
+                ) from e
             else:
                 raise SessionSetupFailure(
-                    errlines[-1].decode(), errlines=errlines) from e
+                    errlines[-1].decode(), errlines=errlines
+                ) from e
         logging.info(
             "Opened schroot session %s (from %s)", self.session_id, self.chroot
         )
@@ -118,10 +138,10 @@ class SchrootSession(Session):
 
     def _run_argv(
         self,
-        argv: List[str],
+        argv: list[str],
         cwd: Optional[str] = None,
         user: Optional[str] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
     ):
         if self.session_id is None:
             raise NoSessionOpen(self)
@@ -138,7 +158,7 @@ class SchrootSession(Session):
                 "-c",
                 " ".join(
                     [
-                        "%s=%s " % (key, shlex.quote(value))
+                        f"{key}={shlex.quote(value)} "
                         for (key, value) in env.items()
                     ]
                     + [shlex.quote(arg) for arg in argv]
@@ -148,10 +168,10 @@ class SchrootSession(Session):
 
     def check_call(
         self,
-        argv: List[str],
+        argv: list[str],
         cwd: Optional[str] = None,
         user: Optional[str] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
         close_fds: bool = True,
     ):
         try:
@@ -163,34 +183,41 @@ class SchrootSession(Session):
 
     def check_output(
         self,
-        argv: List[str],
+        argv: list[str],
         cwd: Optional[str] = None,
         user: Optional[str] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
     ) -> bytes:
         try:
             return subprocess.check_output(
-                self._run_argv(argv, cwd, user, env=env))
+                self._run_argv(argv, cwd, user, env=env)
+            )
         except subprocess.CalledProcessError as e:
             raise subprocess.CalledProcessError(e.returncode, argv) from e
 
     def Popen(
-        self, argv, cwd: Optional[str] = None, user: Optional[str] = None,
-        **kwargs
+        self,
+        argv,
+        cwd: Optional[str] = None,
+        user: Optional[str] = None,
+        **kwargs,
     ):
         return subprocess.Popen(self._run_argv(argv, cwd, user), **kwargs)
 
     def call(
-        self, argv: List[str], cwd: Optional[str] = None,
-        user: Optional[str] = None
+        self,
+        argv: list[str],
+        cwd: Optional[str] = None,
+        user: Optional[str] = None,
     ):
         return subprocess.call(self._run_argv(argv, cwd, user))
 
     def create_home(self) -> None:
         """Create the user's home directory."""
         home = (
-            self.check_output(
-                ["sh", "-c", "echo $HOME"], cwd="/").decode().rstrip("\n")
+            self.check_output(["sh", "-c", "echo $HOME"], cwd="/")
+            .decode()
+            .rstrip("\n")
         )
         user = (
             self.check_output(["sh", "-c", "echo $LOGNAME"], cwd="/")
@@ -207,7 +234,8 @@ class SchrootSession(Session):
         if self._cwd is None:
             raise ValueError("no cwd set")
         return os.path.join(
-            self.location, os.path.join(self._cwd, path).lstrip("/"))
+            self.location, os.path.join(self._cwd, path).lstrip("/")
+        )
 
     def exists(self, path: str) -> bool:
         fullpath = self.external_path(path)
@@ -223,19 +251,25 @@ class SchrootSession(Session):
 
     def rmtree(self, path: str):
         import shutil
+
         fullpath = self.external_path(path)
         return shutil.rmtree(fullpath)
 
+    def _build_tempdir(self, build_dir="/build") -> str:
+        return (
+            self.check_output(["mktemp", "-d", "-p", build_dir], cwd="/")
+            .decode()
+            .rstrip("\n")
+        )
+
     def setup_from_vcs(
         self, tree, include_controldir: Optional[bool] = None, subdir="package"
     ):
         from ..vcs import dupe_vcs_tree, export_vcs_tree
 
-        build_dir = os.path.join(self.location, "build")
-        directory = tempfile.mkdtemp(dir=build_dir)
-        reldir = "/" + os.path.relpath(directory, self.location)
+        reldir = self._build_tempdir()
 
-        export_directory = os.path.join(directory, subdir)
+        export_directory = os.path.join(self.external_path(reldir), subdir)
         if not include_controldir:
             export_vcs_tree(tree, export_directory)
         else:
@@ -246,10 +280,8 @@ class SchrootSession(Session):
     def setup_from_directory(self, path, subdir="package"):
         import shutil
 
-        build_dir = os.path.join(self.location, "build")
-        directory = tempfile.mkdtemp(dir=build_dir)
-        reldir = "/" + os.path.relpath(directory, self.location)
-        export_directory = os.path.join(directory, subdir)
+        reldir = self._build_tempdir()
+        export_directory = os.path.join(self.external_path(reldir), subdir)
         shutil.copytree(path, export_directory, symlinks=True)
         return export_directory, os.path.join(reldir, subdir)
 
