@@ -18,36 +18,37 @@
 import errno
 
 from breezy.errors import NotBranchError
-from breezy.export import export
+from breezy.revisiontree import RevisionTree
+from breezy.tree import Tree
 from breezy.workingtree import WorkingTree
-
 from buildlog_consultant.sbuild import (
     NoSpaceOnDevice,
 )
 
 from . import DetailedFailure
+from ._ognibuild_rs import (
+    export_vcs_tree,
+)
+
+__all__ = [
+    "export_vcs_tree",
+]
 
 
-def export_vcs_tree(tree, directory, subpath=""):
-    try:
-        export(tree, directory, "dir", None, subdir=(subpath or None))
-    except OSError as e:
-        if e.errno == errno.ENOSPC:
-            raise DetailedFailure(1, ["export"], NoSpaceOnDevice())
-        raise
-
-
-def dupe_vcs_tree(tree, directory):
+def dupe_vcs_tree(tree: Tree, directory: str) -> None:
     with tree.lock_read():
         if isinstance(tree, WorkingTree):
             tree = tree.basis_tree()
+    assert isinstance(tree, RevisionTree)
     try:
         result = tree._repository.controldir.sprout(
-            directory, create_tree_if_local=True, revision_id=tree.get_revision_id()
+            directory,
+            create_tree_if_local=True,
+            revision_id=tree.get_revision_id(),
         )
     except OSError as e:
         if e.errno == errno.ENOSPC:
-            raise DetailedFailure(1, ["sprout"], NoSpaceOnDevice())
+            raise DetailedFailure(1, ["sprout"], NoSpaceOnDevice()) from e
         raise
     if not result.has_workingtree():
         raise AssertionError
